----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: scalp_firmware - arch
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2020.2
-- Description: scalp_firmware
--
-- Last update: 2021-01-11
--
---------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
-- Aurora packages
use work.aurora_status_pkg.all;
use work.aurora_drp_pkg.all;
-- Axi4 packages
use work.axi4_pkg.all;
-- Scalp
use work.scalp_misc.all;
use work.scalp_utility.all;
--use work.scalp_sim_packets.all;

library unisim;
use unisim.vcomponents.all;

entity scalp_firmware is

    generic (
        C_USE_IBERT                 : boolean                := false;
        C_DEBUG_MODE                : boolean                := false;
        C_RX_FIFO_MODE              : boolean                := true;
        C_SCALP_NUMBER_OF_INTERFACE : integer range 0 to 255 := 7;
        C_SCALP_SCHEDULER_STRATEGY  : string                 := "RR");

    port (
        -----------------------------------------------------------------------
        -- Names defined and not described in the constraint file.
        -----------------------------------------------------------------------
        -- Zynq FIXED_IO
        PSClkxCIO          : inout std_logic;
        PSSRstxRNIO        : inout std_logic;
        PSPorxSNIO         : inout std_logic;
        -- DDR interface
        DDRClkNxCIO        : inout std_logic;
        DDRClkPxCIO        : inout std_logic;
        DDRDRstxRNIO       : inout std_logic;
        DDRCasNxSIO        : inout std_logic;
        DDRCkexSIO         : inout std_logic;
        DDRCsNxSIO         : inout std_logic;
        DDROdtxSIO         : inout std_logic;
        DDRRasNxSIO        : inout std_logic;
        DDRWexSNIO         : inout std_logic;
        DDRBankAddrxDIO    : inout std_logic_vector(2 downto 0);
        DDRAddrxDIO        : inout std_logic_vector(14 downto 0);
        DDRVrNxSIO         : inout std_logic;
        DDRVrPxSIO         : inout std_logic;
        DDRDmxDIO          : inout std_logic_vector(3 downto 0);
        DDRDqxDIO          : inout std_logic_vector(31 downto 0);
        DDRDqsNxDIO        : inout std_logic_vector(3 downto 0);
        DDRDqsPxDIO        : inout std_logic_vector(3 downto 0);
        -- MIO Interface
        MIOxDIO            : inout std_logic_vector(53 downto 0);
        -----------------------------------------------------------------------
        -- USB signals
        UsbVbusPwrFaultxSI : in    std_logic;
        -- PLL interface
        Pll2V5ClkuWirexCO  : out   std_logic;  -- Clock (from SPI1_SCLK)
        Pll2V5DatauWirexSO : out   std_logic;  -- Data (from SPI1_MOSI)
        Pll2V5LEuWirexSO   : out   std_logic;  -- Latch enable (from SPI1_SS)
        Pll2V5GOExSO       : out   std_logic;  -- Global Output Enable
        Pll2V5LDxSI        : in    std_logic;  -- Lock Detect
        Pll2V5SyncxSO      : out   std_logic;  -- Sync
        Pll2V5ClkIn0LOSxSI : in    std_logic;  -- FPGA clock Loss of Sync
        Pll2V5ClkIn1LOSxSI : in    std_logic;  -- External oscillator Loss of Sync
        -- GTP interfaces
        -- Clocks
        GTPRefClk0PxCI     : in    std_logic;
        GTPRefClk0NxCI     : in    std_logic;
        GTPRefClk1PxCI     : in    std_logic;
        GTPRefClk1NxCI     : in    std_logic;
        -- North
        GTPFromNorthPxSI   : in    std_logic;
        GTPFromNorthNxSI   : in    std_logic;
        GTPToNorthPxSO     : out   std_logic;
        GTPToNorthNxSO     : out   std_logic;
        -- East
        GTPFromEastPxSI    : in    std_logic;
        GTPFromEastNxSI    : in    std_logic;
        GTPToEastPxSO      : out   std_logic;
        GTPToEastNxSO      : out   std_logic;
        -- South
        GTPFromSouthPxSI   : in    std_logic;
        GTPFromSouthNxSI   : in    std_logic;
        GTPToSouthPxSO     : out   std_logic;
        GTPToSouthNxSO     : out   std_logic;
        -- West
        GTPFromWestPxSI    : in    std_logic;
        GTPFromWestNxSI    : in    std_logic;
        GTPToWestPxSO      : out   std_logic;
        GTPToWestNxSO      : out   std_logic;
        -- LVDS links towards edge connectors
        -- North
        -- LVDS2V5North0PxSIO  : inout std_logic;
        -- LVDS2V5North0NxSIO  : inout std_logic;
        -- LVDS2V5North1PxSIO  : inout std_logic;
        -- LVDS2V5North1NxSIO  : inout std_logic;
        -- LVDS2V5North2PxSIO  : inout std_logic;
        -- LVDS2V5North2NxSIO  : inout std_logic;
        -- LVDS2V5North3PxSIO  : inout std_logic;
        -- LVDS2V5North3NxSIO  : inout std_logic;
        -- LVDS2V5North4PxSIO  : inout std_logic;
        -- LVDS2V5North4NxSIO  : inout std_logic;
        -- LVDS2V5North5PxSIO  : inout std_logic;
        -- LVDS2V5North5NxSIO  : inout std_logic;
        -- LVDS2V5North6PxSIO  : inout std_logic;
        -- LVDS2V5North6NxSIO  : inout std_logic;
        -- LVDS2V5North7PxSIO  : inout std_logic;
        -- LVDS2V5North7NxSIO  : inout std_logic;
        -- South
        -- LVDS2V5South0PxSIO  : inout std_logic;
        -- LVDS2V5South0NxSIO  : inout std_logic;
        -- LVDS2V5South1PxSIO  : inout std_logic;
        -- LVDS2V5South1NxSIO  : inout std_logic;
        -- LVDS2V5South2PxSIO  : inout std_logic;
        -- LVDS2V5South2NxSIO  : inout std_logic;
        -- LVDS2V5South3PxSIO  : inout std_logic;
        -- LVDS2V5South3NxSIO  : inout std_logic;
        -- LVDS2V5South4PxSIO  : inout std_logic;
        -- LVDS2V5South4NxSIO  : inout std_logic;
        -- LVDS2V5South5PxSIO  : inout std_logic;
        -- LVDS2V5South5NxSIO  : inout std_logic;
        -- LVDS2V5South6PxSIO  : inout std_logic;
        -- LVDS2V5South6NxSIO  : inout std_logic;
        -- LVDS2V5South7PxSIO  : inout std_logic;
        -- LVDS2V5South7NxSIO  : inout std_logic;
        -- East
        -- LVDS2V5East0PxSIO   : inout std_logic;
        -- LVDS2V5East0NxSIO   : inout std_logic;
        -- LVDS2V5East1PxSIO   : inout std_logic;
        -- LVDS2V5East1NxSIO   : inout std_logic;
        -- LVDS2V5East2PxSIO   : inout std_logic;
        -- LVDS2V5East2NxSIO   : inout std_logic;
        -- LVDS2V5East3PxSIO   : inout std_logic;
        -- LVDS2V5East3NxSIO   : inout std_logic;
        -- LVDS2V5East4PxSIO   : inout std_logic;
        -- LVDS2V5East4NxSIO   : inout std_logic;
        -- LVDS2V5East5PxSIO   : inout std_logic;
        -- LVDS2V5East5NxSIO   : inout std_logic;
        -- LVDS2V5East6PxSIO   : inout std_logic;
        -- LVDS2V5East6NxSIO   : inout std_logic;
        -- LVDS2V5East7PxSIO   : inout std_logic;
        -- LVDS2V5East7NxSIO   : inout std_logic;
        -- West
        -- LVDS2V5West0PxSIO   : inout std_logic;
        -- LVDS2V5West0NxSIO   : inout std_logic;
        -- LVDS2V5West1PxSIO   : inout std_logic;
        -- LVDS2V5West1NxSIO   : inout std_logic;
        -- LVDS2V5West2PxSIO   : inout std_logic;
        -- LVDS2V5West2NxSIO   : inout std_logic;
        -- LVDS2V5West3PxSIO   : inout std_logic;
        -- LVDS2V5West3NxSIO   : inout std_logic;
        -- LVDS2V5West4PxSIO   : inout std_logic;
        -- LVDS2V5West4NxSIO   : inout std_logic;
        -- LVDS2V5West5PxSIO   : inout std_logic;
        -- LVDS2V5West5NxSIO   : inout std_logic;
        -- LVDS2V5West6PxSIO   : inout std_logic;
        -- LVDS2V5West6NxSIO   : inout std_logic;
        -- LVDS2V5West7PxSIO   : inout std_logic;
        -- LVDS2V5West7NxSIO   : inout std_logic;
        -- LVDS links towards top-bottom connectors
        -- Top
        -- LVDS2V5Top0PxSIO    : inout std_logic;
        -- LVDS2V5Top0NxSIO    : inout std_logic;
        -- LVDS2V5Top1PxSIO    : inout std_logic;
        -- LVDS2V5Top1NxSIO    : inout std_logic;
        -- LVDS2V5Top2PxSIO    : inout std_logic;
        -- LVDS2V5Top2NxSIO    : inout std_logic;
        -- LVDS2V5Top3PxSIO    : inout std_logic;
        -- LVDS2V5Top3NxSIO    : inout std_logic;
        -- LVDS2V5Top4PxSIO    : inout std_logic;
        -- LVDS2V5Top4NxSIO    : inout std_logic;
        -- LVDS2V5Top5PxSIO    : inout std_logic;
        -- LVDS2V5Top5NxSIO    : inout std_logic;
        -- LVDS2V5Top6PxSIO    : inout std_logic;
        -- LVDS2V5Top6NxSIO    : inout std_logic;
        -- LVDS2V5Top7PxSIO    : inout std_logic;
        -- LVDS2V5Top7NxSIO    : inout std_logic;
        -- Bottom
        -- LVDS2V5Bottom0PxSIO : inout std_logic;
        -- LVDS2V5Bottom0NxSIO : inout std_logic;
        -- LVDS2V5Bottom1PxSIO : inout std_logic;
        -- LVDS2V5Bottom1NxSIO : inout std_logic;
        -- LVDS2V5Bottom2PxSIO : inout std_logic;
        -- LVDS2V5Bottom2NxSIO : inout std_logic;
        -- LVDS2V5Bottom3PxSIO : inout std_logic;
        -- LVDS2V5Bottom3NxSIO : inout std_logic;
        -- LVDS2V5Bottom4PxSIO : inout std_logic;
        -- LVDS2V5Bottom4NxSIO : inout std_logic;
        -- LVDS2V5Bottom5PxSIO : inout std_logic;
        -- LVDS2V5Bottom5NxSIO : inout std_logic;
        -- LVDS2V5Bottom6PxSIO : inout std_logic;
        -- LVDS2V5Bottom6NxSIO : inout std_logic;
        -- LVDS2V5Bottom7PxSIO : inout std_logic;
        -- LVDS2V5Bottom7NxSIO : inout std_logic;
        -- RGB LEDs
        Led12V5RxSO        : out   std_logic;
        Led12V5GxSO        : out   std_logic;
        Led12V5BxSO        : out   std_logic;
        Led22V5RxSO        : out   std_logic;
        Led22V5GxSO        : out   std_logic;
        Led22V5BxSO        : out   std_logic;
        -- Self reset (connected to PS_SRSTB)
        SelfRstxRNO        : out   std_logic);
    -- Clocks from PLLs (connected to MRCC pins)
    -- Local
    -- PLLClk2V5LocalPxCI  : in    std_logic;
    -- PLLClk2V5LocalNxCI  : in    std_logic;
    -- -- North
    -- PLLClk2V5NorthPxCI  : in    std_logic;
    -- PLLClk2V5NorthNxCI  : in    std_logic;
    -- -- South
    -- PLLClk2V5SouthPxCI  : in    std_logic;
    -- PLLClk2V5SouthNxCI  : in    std_logic;
    -- -- Top
    -- PLLClk2V5TopxCI     : in    std_logic;  -- Single-ended
    -- -- Bottom
    -- PLLClk2V5BottomxCI  : in    std_logic;  -- Single-ended
    -- -- Clocks to/from neighbours
    -- -- North
    -- Clk2V5NorthPxCI     : in    std_logic;
    -- Clk2V5NorthNxCI     : in    std_logic;
    -- Clk2V5NorthPxCO     : out   std_logic;
    -- Clk2V5NorthNxCO     : out   std_logic;
    -- -- South
    -- Clk2V5SouthPxCI     : in    std_logic;
    -- Clk2V5SouthNxCI     : in    std_logic;
    -- Clk2V5SouthPxCO     : out   std_logic;
    -- Clk2V5SouthNxCO     : out   std_logic;
    -- -- East
    -- Clk2V5EastPxCI      : in    std_logic;
    -- Clk2V5EastNxCI      : in    std_logic;
    -- Clk2V5EastPxCO      : out   std_logic;
    -- Clk2V5EastNxCO      : out   std_logic;
    -- -- West
    -- Clk2V5WestPxCI      : in    std_logic;
    -- Clk2V5WestNxCI      : in    std_logic;
    -- Clk2V5WestPxCO      : out   std_logic;
    -- Clk2V5WestNxCO      : out   std_logic;
    -- -- Top
    -- Clk2V5TopPxCI       : in    std_logic;
    -- Clk2V5TopNxCI       : in    std_logic;
    -- Clk2V5TopPxCO       : out   std_logic;
    -- Clk2V5TopNxCO       : out   std_logic;
    -- -- Bottom
    -- Clk2V5BottomPxCI    : in    std_logic;
    -- Clk2V5BottomNxCI    : in    std_logic;
    -- Clk2V5BottomPxCO    : out   std_logic;
    -- Clk2V5BottomNxCO    : out   std_logic;
    -- -- Recovery
    -- Clk2V5RecoveryPxCO  : out   std_logic;
    -- Clk2V5RecoveryNxCO  : out   std_logic);

end scalp_firmware;


architecture arch of scalp_firmware is

    -- Constantes
    -- constant C_PS_SYS_RESET_SIZE : integer range 0 to 7 := 1;
    constant C_AXI_ADDR_SIZE : integer range 0 to 32 := 12;

    component scalp_aurora_phy is
        generic (
            C_DEBUG_MODE   : boolean;
            C_RX_FIFO_MODE : boolean);
        port (
            GTRefClkxCI          : in  t_gt_ref_slave_clk;
            AuroraClkxCI         : in  t_aurora_slave_clk;
            AuroraClkxCO         : out t_aurora_master_clk;
            AuroraResetxRI       : in  t_aurora_slave_reset;
            AuroraResetxRO       : out t_aurora_master_link_reset;
            RXResetxRI           : in  t_rx_reset;
            RXFifoResetDonexDO   : out t_rx_fifo_reset_done;
            GTPFromNorthxDI      : in  t_aurora_gtp_diff_io_rx;
            GTPToNorthxDO        : out t_aurora_gtp_diff_io_tx;
            GTPFromEastxDI       : in  t_aurora_gtp_diff_io_rx;
            GTPToEastxDO         : out t_aurora_gtp_diff_io_tx;
            GTPFromSouthxDI      : in  t_aurora_gtp_diff_io_rx;
            GTPToSouthxDO        : out t_aurora_gtp_diff_io_tx;
            GTPFromWestxDI       : in  t_aurora_gtp_diff_io_rx;
            GTPToWestxDO         : out t_aurora_gtp_diff_io_tx;
            NorthRXM2SxDO        : out t_axi4m2s;
            NorthRXS2MxDI        : in  t_axi4s2m;
            NorthTXM2SxDI        : in  t_axi4m2s;
            NorthTXS2MxDO        : out t_axi4s2m;
            EastRXM2SxDO         : out t_axi4m2s;
            EastRXS2MxDI         : in  t_axi4s2m;
            EastTXM2SxDI         : in  t_axi4m2s;
            EastTXS2MxDO         : out t_axi4s2m;
            SouthRXM2SxDO        : out t_axi4m2s;
            SouthRXS2MxDI        : in  t_axi4s2m;
            SouthTXM2SxDI        : in  t_axi4m2s;
            SouthTXS2MxDO        : out t_axi4s2m;
            WestRXM2SxDO         : out t_axi4m2s;
            WestRXS2MxDI         : in  t_axi4s2m;
            WestTXM2SxDI         : in  t_axi4m2s;
            WestTXS2MxDO         : out t_axi4s2m;
            NorthRXUFCM2SxDO     : out t_axi4ufcm2s_rx;
            NorthTXUFCM2SxDI     : in  t_axi4ufcm2s_tx;
            NorthTXUFCS2MxDO     : out t_axi4ufcs2m_tx;
            EastRXUFCM2SxDO      : out t_axi4ufcm2s_rx;
            EastTXUFCM2SxDI      : in  t_axi4ufcm2s_tx;
            EastTXUFCS2MxDO      : out t_axi4ufcs2m_tx;
            SouthRXUFCM2SxDO     : out t_axi4ufcm2s_rx;
            SouthTXUFCM2SxDI     : in  t_axi4ufcm2s_tx;
            SouthTXUFCS2MxDO     : out t_axi4ufcs2m_tx;
            WestRXUFCM2SxDO      : out t_axi4ufcm2s_rx;
            WestTXUFCM2SxDI      : in  t_axi4ufcm2s_tx;
            WestTXUFCS2MxDO      : out t_axi4ufcs2m_tx;
            NorthRXNFCM2SxDO     : out t_axi4nfcm2s;
            NorthTXNFCM2SxDI     : in  t_axi4nfcm2s;
            NorthTXNFCS2MxDO     : out t_axi4nfcs2m;
            EastRXNFCM2SxDO      : out t_axi4nfcm2s;
            EastTXNFCM2SxDI      : in  t_axi4nfcm2s;
            EastTXNFCS2MxDO      : out t_axi4nfcs2m;
            SouthRXNFCM2SxDO     : out t_axi4nfcm2s;
            SouthTXNFCM2SxDI     : in  t_axi4nfcm2s;
            SouthTXNFCS2mxDO     : out t_axi4nfcs2m;
            WestRXNFCM2SxDO      : out t_axi4nfcm2s;
            WestTXNFCM2SxDI      : in  t_axi4nfcm2s;
            WestTXNFCS2MxDO      : out t_axi4nfcs2m;
            AuroraCtrlxDI        : in  t_aurora_control;
            AuroraStatusxDO      : out t_aurora_status;
            AuroraDRPM2SxDI      : in  t_drpm2s_vector((C_NB_GTP_CORE - 1) downto 0);
            AuroraDRPS2MxDO      : out t_drps2m_vector((C_NB_GTP_CORE - 1) downto 0);
            NorthRXFifoStatusxDO : out t_axi4fifo_status;
            EastRXFifoStatusxDO  : out t_axi4fifo_status;
            SouthRXFifoStatusxDO : out t_axi4fifo_status;
            WestRXFifoStatusxDO  : out t_axi4fifo_status;
            AxisFifoErrorxDO     : out t_axi4fifo_error);
    end component scalp_aurora_phy;

    -- Signals
    -- Clocks
    -- Processing system clock
    signal PSSysClkxC               : std_logic                                                      := '0';
    -- GTP Clocks
    -- signal GTPRefClk0xC : std_logic := '0';
    -- signal GTPRefClk1xC : std_logic := '0';
    signal GTRefClk0DiffxC          : t_gtp_diff_ref_clk                                             := C_NO_GTP_DIFF_REF_CLK;
    signal GTRefClk1DiffxC          : t_gtp_diff_ref_clk                                             := C_NO_GTP_DIFF_REF_CLK;
    -- Resets
    -- Processing system reset
    signal PSSysResetxR             : std_logic                                                      := '0';
    -- Scalp Aurora Phy
    signal GTRefClk0xC              : t_gt_ref_slave_clk                                             := C_GT_REF_NO_SLAVE_CLK;
    signal GTRefClk1xC              : t_gt_ref_slave_clk                                             := C_GT_REF_NO_SLAVE_CLK;
    signal AuroraClkSlavexC         : t_aurora_slave_clk                                             := C_AURORA_NO_SLAVE_CLK;
    signal AuroraClkMasterxC        : t_aurora_master_clk                                            := C_AURORA_NO_MASTER_CLK;
    signal AuroraResetSlavexR       : t_aurora_slave_reset                                           := C_AURORA_NO_SLAVE_RESET;
    signal AuroraResetMasterLinkxR  : t_aurora_master_link_reset                                     := C_AURORA_NO_MASTER_LINK_RESET;
    signal RXResetxR                : t_rx_reset                                                     := C_NO_RX_RESET;
    signal RXFifoResetDonexD        : t_rx_fifo_reset_done                                           := C_NO_RX_FIFO_RESET_DONE;
    signal RXFifoResetDoneDelayedxD : t_rx_fifo_reset_done                                           := C_NO_RX_FIFO_RESET_DONE;
    signal GTPFromNorthxD           : t_aurora_gtp_diff_io_rx                                        := C_AURORA_NO_GTP_DIFF_IO_RX;
    signal GTPToNorthxD             : t_aurora_gtp_diff_io_tx                                        := C_AURORA_NO_GTP_DIFF_IO_TX;
    signal GTPFromEastxD            : t_aurora_gtp_diff_io_rx                                        := C_AURORA_NO_GTP_DIFF_IO_RX;
    signal GTPToEastxD              : t_aurora_gtp_diff_io_tx                                        := C_AURORA_NO_GTP_DIFF_IO_TX;
    signal GTPFromSouthxD           : t_aurora_gtp_diff_io_rx                                        := C_AURORA_NO_GTP_DIFF_IO_RX;
    signal GTPToSouthxD             : t_aurora_gtp_diff_io_tx                                        := C_AURORA_NO_GTP_DIFF_IO_TX;
    signal GTPFromWestxD            : t_aurora_gtp_diff_io_rx                                        := C_AURORA_NO_GTP_DIFF_IO_RX;
    signal GTPToWestxD              : t_aurora_gtp_diff_io_tx                                        := C_AURORA_NO_GTP_DIFF_IO_TX;
    signal NorthRXM2SxD             : t_axi4m2s                                                      := C_NO_AXI4_M2S;
    signal NorthRXS2MxD             : t_axi4s2m                                                      := C_NO_AXI4_S2M;
    signal NorthTXM2SxD             : t_axi4m2s                                                      := C_NO_AXI4_M2S;
    signal NorthTXS2MxD             : t_axi4s2m                                                      := C_NO_AXI4_S2M;
    signal EastRXM2SxD              : t_axi4m2s                                                      := C_NO_AXI4_M2S;
    signal EastRXS2MxD              : t_axi4s2m                                                      := C_NO_AXI4_S2M;
    signal EastTXM2SxD              : t_axi4m2s                                                      := C_NO_AXI4_M2S;
    signal EastTXS2MxD              : t_axi4s2m                                                      := C_NO_AXI4_S2M;
    signal SouthRXM2SxD             : t_axi4m2s                                                      := C_NO_AXI4_M2S;
    signal SouthRXS2MxD             : t_axi4s2m                                                      := C_NO_AXI4_S2M;
    signal SouthTXM2SxD             : t_axi4m2s                                                      := C_NO_AXI4_M2S;
    signal SouthTXS2MxD             : t_axi4s2m                                                      := C_NO_AXI4_S2M;
    signal WestRXM2SxD              : t_axi4m2s                                                      := C_NO_AXI4_M2S;
    signal WestRXS2MxD              : t_axi4s2m                                                      := C_NO_AXI4_S2M;
    signal WestTXM2SxD              : t_axi4m2s                                                      := C_NO_AXI4_M2S;
    signal WestTXS2MxD              : t_axi4s2m                                                      := C_NO_AXI4_S2M;
    signal NorthRXUFCM2SxD          : t_axi4ufcm2s_rx                                                := C_NO_AXI4_UFC_M2S_RX;
    signal NorthTXUFCM2SxD          : t_axi4ufcm2s_tx                                                := C_NO_AXI4_UFC_M2S_TX;
    signal NorthTXUFCS2MxD          : t_axi4ufcs2m_tx                                                := C_NO_AXI4_UFC_S2M_TX;
    signal EastRXUFCM2SxD           : t_axi4ufcm2s_rx                                                := C_NO_AXI4_UFC_M2S_RX;
    signal EastTXUFCM2SxD           : t_axi4ufcm2s_tx                                                := C_NO_AXI4_UFC_M2S_TX;
    signal EastTXUFCS2MxD           : t_axi4ufcs2m_tx                                                := C_NO_AXI4_UFC_S2M_TX;
    signal SouthRXUFCM2SxD          : t_axi4ufcm2s_rx                                                := C_NO_AXI4_UFC_M2S_RX;
    signal SouthTXUFCM2SxD          : t_axi4ufcm2s_tx                                                := C_NO_AXI4_UFC_M2S_TX;
    signal SouthTXUFCS2MxD          : t_axi4ufcs2m_tx                                                := C_NO_AXI4_UFC_S2M_TX;
    signal WestRXUFCM2SxD           : t_axi4ufcm2s_rx                                                := C_NO_AXI4_UFC_M2S_RX;
    signal WestTXUFCM2SxD           : t_axi4ufcm2s_tx                                                := C_NO_AXI4_UFC_M2S_TX;
    signal WestTXUFCS2MxD           : t_axi4ufcs2m_tx                                                := C_NO_AXI4_UFC_S2M_TX;
    signal NorthRXNFCM2SxD          : t_axi4nfcm2s                                                   := C_NO_AXI4_NFC_M2S;
    signal NorthTXNFCM2SxD          : t_axi4nfcm2s                                                   := C_NO_AXI4_NFC_M2S;
    signal NorthTXNFCS2MxD          : t_axi4nfcs2m                                                   := C_NO_AXI4_NFC_S2M;
    signal EastRXNFCM2SxD           : t_axi4nfcm2s                                                   := C_NO_AXI4_NFC_M2S;
    signal EastTXNFCM2SxD           : t_axi4nfcm2s                                                   := C_NO_AXI4_NFC_M2S;
    signal EastTXNFCS2MxD           : t_axi4nfcs2m                                                   := C_NO_AXI4_NFC_S2M;
    signal SouthRXNFCM2SxD          : t_axi4nfcm2s                                                   := C_NO_AXI4_NFC_M2S;
    signal SouthTXNFCM2SxD          : t_axi4nfcm2s                                                   := C_NO_AXI4_NFC_M2S;
    signal SouthTXNFCS2MxD          : t_axi4nfcs2m                                                   := C_NO_AXI4_NFC_S2M;
    signal WestRXNFCM2SxD           : t_axi4nfcm2s                                                   := C_NO_AXI4_NFC_M2S;
    signal WestTXNFCM2SxD           : t_axi4nfcm2s                                                   := C_NO_AXI4_NFC_M2S;
    signal WestTXNFCS2MxD           : t_axi4nfcs2m                                                   := C_NO_AXI4_NFC_S2M;
    signal AuroraCtrlxD             : t_aurora_control                                               := C_AURORA_NO_CONTROL;
    signal AuroraStatusxD           : t_aurora_status                                                := C_AURORA_NO_STATUS;
    signal AuroraDRPM2SxD           : t_drpm2s_vector((C_NB_GTP_CORE - 1) downto 0)                  := (others => C_NO_DRP_M2S);
    signal AuroraDRPS2MxD           : t_drps2m_vector((C_NB_GTP_CORE - 1) downto 0)                  := (others => C_NO_DRP_S2M);
    signal NorthRXFifoStatusxD      : t_axi4fifo_status                                              := C_NO_AXI4_FIFO_STATUS;
    signal EastRXFifoStatusxD       : t_axi4fifo_status                                              := C_NO_AXI4_FIFO_STATUS;
    signal SouthRXFifoStatusxD      : t_axi4fifo_status                                              := C_NO_AXI4_FIFO_STATUS;
    signal WestRXFifoStatusxD       : t_axi4fifo_status                                              := C_NO_AXI4_FIFO_STATUS;
    signal AxisFifoErrorxD          : t_axi4fifo_error                                               := C_NO_AXI4_FIFO_ERROR;
    -- Scalp Router
    signal LocNetAddrxD             : t_scalp_netaddr                                                := C_3D_MIN_SCALP_NETADDR;
    signal RXAxism2sVectorxD        : t_axi4m2s_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)   := (others => C_NO_AXI4_M2S);
    signal RXAxiss2mVectorxD        : t_axi4s2m_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)   := (others => C_NO_AXI4_S2M);
    signal TXAxism2sVectorxD        : t_axi4m2s_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)   := (others => C_NO_AXI4_M2S);
    signal TXAxiss2mVectorxD        : t_axi4s2m_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)   := (others => C_NO_AXI4_S2M);
    signal QoSVectorxD              : t_scalp_qos_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0) := (others => C_SCALP_NO_QOS);
    -- signal ScalpRouterReadyxD       : t_scalp_router_ready                                           := C_NO_SCALP_ROUTER_READY;
    -- Scalp Axi Lite interface and IRQ
    signal InterruptxS              : std_ulogic                                                     := '0';
    signal RdAddrxD                 : std_ulogic_vector((C_AXI_ADDR_SIZE - 1) downto 0)              := (others => '0');
    signal RdDataxD                 : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0)             := (others => '0');
    signal RdValidxS                : std_ulogic                                                     := '0';
    signal WrAddrxD                 : std_ulogic_vector((C_AXI_ADDR_SIZE - 1) downto 0)              := (others => '0');
    signal WrDataxD                 : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0)             := (others => '0');
    signal WrValidxS                : std_ulogic                                                     := '0';
    -- Zynq Reg Bank
    -- type t_status_send_word is (E_IDLE, E_SEND);

    -- signal NorthStatusSendWordxDN   : t_status_send_word                                 := E_IDLE;
    -- signal NorthStatusSendWordxDP   : t_status_send_word                                 := E_IDLE;
    -- signal EastStatusSendWordxDN    : t_status_send_word                                 := E_IDLE;
    -- signal EastStatusSendWordxDP    : t_status_send_word                                 := E_IDLE;
    -- signal SouthStatusSendWordxDN   : t_status_send_word                                 := E_IDLE;
    -- signal SouthStatusSendWordxDP   : t_status_send_word                                 := E_IDLE;
    -- signal WestStatusSendWordxDN    : t_status_send_word                                 := E_IDLE;
    -- signal WestStatusSendWordxDP    : t_status_send_word                                 := E_IDLE;
    -- --
    -- signal NorthNativeSlavexD       : t_native_fifo_slave;
    -- signal NorthNativeMasterxD      : t_native_fifo_master;
    -- signal EastNativeSlavexD        : t_native_fifo_slave;
    -- signal EastNativeMasterxD       : t_native_fifo_master;
    -- signal SouthNativeSlavexD       : t_native_fifo_slave;
    -- signal SouthNativeMasterxD      : t_native_fifo_master;
    -- signal WestNativeSlavexD        : t_native_fifo_slave;
    -- signal WestNativeMasterxD       : t_native_fifo_master;
    --
    -- signal InterruptRegPortxDN    : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal InterruptRegPortxDP    : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- North
    -- signal NorthStatusRegPortxDN    : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal NorthStatusRegPortxDP    : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    signal NorthCtrlRegPortxDN      : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    signal NorthCtrlRegPortxDP      : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal NorthWrDataRegPortxDN    : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal NorthWrDataRegPortxDP    : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- East
    -- signal EastStatusRegPortxDN     : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal EastStatusRegPortxDP     : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal EastCtrlRegPortxDN       : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal EastCtrlRegPortxDP       : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal EastWrDataRegPortxDN     : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal EastWrDataRegPortxDP     : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- South
    -- signal SouthStatusRegPortxDN    : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal SouthStatusRegPortxDP    : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal SouthCtrlRegPortxDN      : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal SouthCtrlRegPortxDP      : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal SouthWrDataRegPortxDN    : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal SouthWrDataRegPortxDP    : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- West
    -- signal WestStatusRegPortxDN     : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal WestStatusRegPortxDP     : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal WestCtrlRegPortxDN       : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal WestCtrlRegPortxDP       : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal WestWrDataRegPortxDN     : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- signal WestWrDataRegPortxDP     : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- Debug
    -- signal CntRstxR               : std_ulogic                                         := '0';
    signal ClkEnxS                  : std_ulogic                                         := '0';
    signal NorthDataCounterxDN      : unsigned(31 downto 0)                              := (others => '0');
    signal NorthDataCounterxDP      : unsigned(31 downto 0)                              := (others => '0');
    signal EastDataCounterxDN       : unsigned(31 downto 0)                              := (others => '0');
    signal EastDataCounterxDP       : unsigned(31 downto 0)                              := (others => '0');
    signal SouthDataCounterxDN      : unsigned(31 downto 0)                              := (others => '0');
    signal SouthDataCounterxDP      : unsigned(31 downto 0)                              := (others => '0');
    signal WestDataCounterxDN       : unsigned(31 downto 0)                              := (others => '0');
    signal WestDataCounterxDP       : unsigned(31 downto 0)                              := (others => '0');
    --
    signal DebugCounterResetxR      : t_if_common_reset                                  := C_NO_IF_COMMON_RESET;
    signal DebugRXFifoResetxR       : t_rx_fifo_reset                                    := C_NO_RX_FIFO_RESET;
    signal DebugBackPressureResetxR : t_rx_back_pressure_reset                           := C_NO_RX_BACK_PRESSURE_RESET;

    -- Attributes
    attribute mark_debug                : string;
    attribute keep                      : string;
    -- Clocks
    attribute keep of PSSysClkxC        : signal is "true";
    attribute keep of GTRefClk0xC       : signal is "true";
    attribute keep of GTRefClk1xC       : signal is "true";
    attribute keep of AuroraClkSlavexC  : signal is "true";
    attribute keep of AuroraClkMasterxC : signal is "true";
    -- North
    -- East
    -- attribute mark_debug of EastRXM2SxD       : signal is "true";
    -- attribute keep of EastRXM2SxD             : signal is "true";
    -- attribute mark_debug of EastRXS2MxD       : signal is "true";
    -- attribute keep of EastRXS2MxD             : signal is "true";
    -- attribute mark_debug of EastTXM2SxD       : signal is "true";
    -- attribute keep of EastTXM2SxD             : signal is "true";
    -- attribute mark_debug of EastTXS2MxD       : signal is "true";
    -- attribute keep of EastTXS2MxD             : signal is "true";
    -- South
    -- West
    -- attribute mark_debug of WestRXM2SxD       : signal is "true";
    -- attribute keep of WestRXM2SxD             : signal is "true";
    -- attribute mark_debug of WestRXS2MxD       : signal is "true";
    -- attribute keep of WestRXS2MxD             : signal is "true";
    -- attribute mark_debug of WestTXM2SxD       : signal is "true";
    -- attribute keep of WestTXM2SxD             : signal is "true";
    -- attribute mark_debug of WestTXS2MxD       : signal is "true";
    -- attribute keep of WestTXS2MxD             : signal is "true";
    -- attribute mark_debug of ScalpRouterReadyxD : signal is "true";
    -- attribute keep of ScalpRouterReadyxD       : signal is "true";
    -- attribute mark_debug of RXAxism2sVectorxD : signal is "true";
    -- attribute keep of RXAxism2sVectorxD       : signal is "true";
    -- attribute mark_debug of RXAxiss2mVectorxD : signal is "true";
    -- attribute keep of RXAxiss2mVectorxD       : signal is "true";
    -- attribute mark_debug of TXAxism2sVectorxD : signal is "true";
    -- attribute keep of TXAxism2sVectorxD       : signal is "true";
    -- attribute mark_debug of TXAxiss2mVectorxD : signal is "true";
    -- attribute keep of TXAxiss2mVectorxD       : signal is "true";

begin

    ProcessingSystemxB : block is
    begin  -- block ProcessingSystemxB

        ZynqxI : entity work.scalp_zynqps_wrapper
            port map (
                -- Processor interface
                FIXED_IO_ps_clk     => PSClkxCIO,
                FIXED_IO_ps_porb    => PSPorxSNIO,
                FIXED_IO_ps_srstb   => PSSRstxRNIO,
                FclkClk0xCO         => PSSysClkxC,
                FclkReset0xRO       => PSSysResetxR,
                -- DDR interface
                DDR_addr            => DDRAddrxDIO,
                DDR_ba              => DDRBankAddrxDIO,
                DDR_cas_n           => DDRCasNxSIO,
                DDR_ck_n            => DDRClkNxCIO,
                DDR_ck_p            => DDRClkPxCIO,
                DDR_cke             => DDRCkexSIO,
                DDR_cs_n            => DDRCsNxSIO,
                DDR_dm              => DDRDmxDIO,
                DDR_dq              => DDRDqxDIO,
                DDR_dqs_n           => DDRDqsNxDIO,
                DDR_dqs_p           => DDRDqsPxDIO,
                DDR_odt             => DDROdtxSIO,
                DDR_ras_n           => DDRRasNxSIO,
                DDR_reset_n         => DDRDRstxRNIO,
                DDR_we_n            => DDRWexSNIO,
                FIXED_IO_ddr_vrn    => DDRVrNxSIO,
                FIXED_IO_ddr_vrp    => DDRVrPxSIO,
                -- USB interface
                Usb0VBusPwrFaultxSI => UsbVbusPwrFaultxSI,
                -- SPI1 used as uWire master. Clk, Data and LE signals are outputs
                -- SPI1 inputs are unused. Clk is connected to SCLK, Data to MOSI and LE to SS
                Spi1MOSIxSO         => Pll2V5DatauWirexSO,
                Spi1SSxSO           => Pll2V5LEuWirexSO,
                Spi1SclkxCO         => Pll2V5ClkuWirexCO,
                -- MIO
                FIXED_IO_mio        => MIOxDIO,
                -- Scalp Axi Lite interface and IRQ
                InterruptxSI        => InterruptxS,
                RdAddrxDO           => RdAddrxD,
                RdDataxDI           => RdDataxD,
                RdValidxSO          => RdValidxS,
                WrAddrxDO           => WrAddrxD,
                WrDataxDO           => WrDataxD,
                WrValidxSO          => WrValidxS);

    end block ProcessingSystemxB;

    ProgrammableLogicxB : block is
    begin  -- block ProgrammableLogicxB        

        InputClocksxB : block is
        begin  -- block InputClocksxB

            GTPRefClk0PxAS : GTRefClk0DiffxC.ClkPxC <= GTPRefClk0PxCI;
            GTPRefClk0NxAS : GTRefClk0DiffxC.ClkNxC <= GTPRefClk0NxCI;
            -- GTPRefClk1PxAS : GTRefClk0DiffxC.ClkPxC <= GTPRefClk1PxCI;
            -- GTPRefClk1NxAS : GTRefClk0DiffxC.ClkNxC <= GTPRefClk1NxCI;

            -- GTP Clocks buffers
            IBufDSGTPRefClk0xI : IBUFDS_GTE2
                port map (
                    I     => GTRefClk0DiffxC.ClkPxC,
                    IB    => GTRefClk0DiffxC.ClkNxC,
                    CEB   => '0',
                    O     => GTRefClk0xC.GTRefClkxC,
                    ODIV2 => open);

            -- IBufDSGTPRefClk1xI : IBUFDS_GTE2
            --     port map (
            --         I     => GTRefClk0DiffxC.ClkPxC,
            --         IB    => GTRefClk0DiffxC.ClkNxC,
            --         CEB   => '0',
            --         O     => GTRefClk1xCGTRefClkxC,
            --         ODIV2 => open);

        end block InputClocksxB;

        PllClocksxB : block is

            constant C_AURORA_RST_DLY_TICKS : integer := 25;
            constant C_GT_RST_DLY_TICKS     : integer := 625;

            signal PllLockedxS : std_ulogic := '0';

        begin  -- block PllClocksxB

            ScalpAuroraPllxI : entity work.scalp_aurora_clk
                port map (
                    -- Clock out ports  
                    InitClkxCO   => AuroraClkSlavexC.InitClkxC,
                    DrpClkxCO    => AuroraClkSlavexC.DrpClkxC,
                    -- Status and control signals                
                    reset        => PSSysResetxR,
                    PllLockedxSO => PllLockedxS,
                    -- Clock in ports
                    PSSysClkxCI  => PSSysClkxC);

            AuroraRstDlyxI : entity work.reset_delay_gen
                generic map (
                    C_TICKS => C_AURORA_RST_DLY_TICKS)
                port map (
                    ClkxCI          => AuroraClkSlavexC.InitClkxC,
                    PllLockedxSI    => PllLockedxS,
                    ResetxRI        => PSSysResetxR,
                    ResetDelayedxRO => AuroraResetSlavexR.ResetxR);

            GTRstDlyxI : entity work.reset_delay_gen
                generic map (
                    C_TICKS => C_GT_RST_DLY_TICKS)
                port map (
                    ClkxCI          => AuroraClkSlavexC.InitClkxC,
                    PllLockedxSI    => PllLockedxS,
                    ResetxRI        => PSSysResetxR,
                    ResetDelayedxRO => AuroraResetSlavexR.GTResetxR);

        end block PllClocksxB;

        GTPhyxB : block is

            constant C_RX_FIFO_RST_DONE_DLY_TICKS : integer := 100;

        begin  -- block GTPhyxB

            -- GTP
            GTPFromNorthPxAS : GTPFromNorthxD.RXPxD(0) <= GTPFromNorthPxSI;
            GTPFromNorthNxAS : GTPFromNorthxD.RXNxD(0) <= GTPFromNorthNxSI;
            GTPToNorthPxAS   : GTPToNorthPxSO          <= GTPToNorthxD.TXPxD(0);
            GTPToNorthNxAS   : GTPToNorthNxSO          <= GTPToNorthxD.TXNxD(0);
            GTPFromEastPxAS  : GTPFromEastxD.RXPxD(0)  <= GTPFromEastPxSI;
            GTPFromEastNxAS  : GTPFromEastxD.RXNxD(0)  <= GTPFromEastNxSI;
            GTPToEastPxAS    : GTPToEastPxSO           <= GTPToEastxD.TXPxD(0);
            GTPToEastNxAS    : GTPToEastNxSO           <= GTPToEastxD.TXNxD(0);
            GTPFromSouthPxAS : GTPFromSouthxD.RXPxD(0) <= GTPFromSouthPxSI;
            GTPFromSouthNxAS : GTPFromSouthxD.RXNxD(0) <= GTPFromSouthNxSI;
            GTPToSouthPxAS   : GTPToSouthPxSO          <= GTPToSouthxD.TXPxD(0);
            GTPToSouthNxAS   : GTPToSouthNxSO          <= GTPToSouthxD.TXNxD(0);
            GTPFromWestPxAS  : GTPFromWestxD.RXPxD(0)  <= GTPFromWestPxSI;
            GTPFromWestNxAS  : GTPFromWestxD.RXNxD(0)  <= GTPFromWestNxSI;
            GTPToWestPxAS    : GTPToWestPxSO           <= GTPToWestxD.TXPxD(0);
            GTPToWestNxAS    : GTPToWestNxSO           <= GTPToWestxD.TXNxD(0);

            CtrlxB : block is
            begin  -- block CtrlxB

                PowerDownxAS : AuroraCtrlxD.PowerDownxS <= '0';
                LoopbackxAS  : AuroraCtrlxD.LoopbackxD  <= (others => '0');

            end block CtrlxB;

            ScalpAuroraPhyxI : entity work.scalp_aurora_phy
                generic map (
                    C_DEBUG_MODE                 => C_DEBUG_MODE,
                    C_RX_FIFO_MODE               => C_RX_FIFO_MODE,
                    C_RX_FIFO_RST_DONE_DLY_TICKS => C_RX_FIFO_RST_DONE_DLY_TICKS)
                port map (
                    -- Clocks
                    -- GTP Ref Clocks
                    GTRefClkxCI               => GTRefClk0xC,
                    -- Aurora System and GTP Clocks
                    AuroraClkxCI              => AuroraClkSlavexC,
                    AuroraClkxCO              => AuroraClkMasterxC,
                    -- Reset
                    -- Aurora Reset
                    AuroraResetxRI            => AuroraResetSlavexR,
                    AuroraResetxRO            => AuroraResetMasterLinkxR,
                    -- RX Fifo and Back Pressure Reset
                    RXResetxRI                => RXResetxR,
                    RXFifoResetDonexDO        => RXFifoResetDonexD,
                    RXFifoResetDoneDelayedxDO => RXFifoResetDoneDelayedxD,
                    -- Back Pressure Reset
                    -- GTP Serial IO
                    -- North
                    GTPFromNorthxDI           => GTPFromNorthxD,
                    GTPToNorthxDO             => GTPToNorthxD,
                    -- East
                    GTPFromEastxDI            => GTPFromEastxD,
                    GTPToEastxDO              => GTPToEastxD,
                    -- South
                    GTPFromSouthxDI           => GTPFromSouthxD,
                    GTPToSouthxDO             => GTPToSouthxD,
                    -- West
                    GTPFromWestxDI            => GTPFromWestxD,
                    GTPToWestxDO              => GTPToWestxD,
                    -- Axi4 Framing Interface
                    -- North
                    NorthRXM2SxDO             => NorthRXM2SxD,
                    NorthRXS2MxDI             => NorthRXS2MxD,
                    NorthTXM2SxDI             => NorthTXM2SxD,
                    NorthTXS2MxDO             => NorthTXS2MxD,
                    -- East
                    EastRXM2SxDO              => EastRXM2SxD,
                    EastRXS2MxDI              => EastRXS2MxD,
                    EastTXM2SxDI              => EastTXM2SxD,
                    EastTXS2MxDO              => EastTXS2MxD,
                    -- South
                    SouthRXM2SxDO             => SouthRXM2SxD,
                    SouthRXS2MxDI             => SouthRXS2MxD,
                    SouthTXM2SxDI             => SouthTXM2SxD,
                    SouthTXS2MxDO             => SouthTXS2MxD,
                    -- West
                    WestRXM2SxDO              => WestRXM2SxD,
                    WestRXS2MxDI              => WestRXS2MxD,
                    WestTXM2SxDI              => WestTXM2SxD,
                    WestTXS2MxDO              => WestTXS2MxD,
                    -- Axi4 Framing UFC Interface
                    -- North
                    NorthRXUFCM2SxDO          => NorthRXUFCM2SxD,
                    NorthTXUFCM2SxDI          => NorthTXUFCM2SxD,
                    NorthTXUFCS2MxDO          => NorthTXUFCS2MxD,
                    -- East
                    EastRXUFCM2SxDO           => EastRXUFCM2SxD,
                    EastTXUFCM2SxDI           => EastTXUFCM2SxD,
                    EastTXUFCS2MxDO           => EastTXUFCS2MxD,
                    -- South
                    SouthRXUFCM2SxDO          => SouthRXUFCM2SxD,
                    SouthTXUFCM2SxDI          => SouthTXUFCM2SxD,
                    SouthTXUFCS2MxDO          => SouthTXUFCS2MxD,
                    -- West
                    WestRXUFCM2SxDO           => WestRXUFCM2SxD,
                    WestTXUFCM2SxDI           => WestTXUFCM2SxD,
                    WestTXUFCS2MxDO           => WestTXUFCS2MxD,
                    -- Axi4 Framing NFC Interface
                    -- The NFC interface is not available when the
                    -- constant C_RX_FIFO_MODE is set to TRUE.
                    -- North
                    NorthRXNFCM2SxDO          => NorthRXNFCM2SxD,
                    NorthTXNFCM2SxDI          => NorthTXNFCM2SxD,
                    NorthTXNFCS2MxDO          => NorthTXNFCS2MxD,
                    -- East
                    EastRXNFCM2SxDO           => EastRXNFCM2SxD,
                    EastTXNFCM2SxDI           => EastTXNFCM2SxD,
                    EastTXNFCS2MxDO           => EastTXNFCS2MxD,
                    -- South
                    SouthRXNFCM2SxDO          => SouthRXNFCM2SxD,
                    SouthTXNFCM2SxDI          => SouthTXNFCM2SxD,
                    SouthTXNFCS2mxDO          => SouthTXNFCS2mxD,
                    -- West
                    WestRXNFCM2SxDO           => WestRXNFCM2SxD,
                    WestTXNFCM2SxDI           => WestTXNFCM2SxD,
                    WestTXNFCS2MxDO           => WestTXNFCS2MxD,
                    -- Aurora Ctrl + Status
                    AuroraCtrlxDI             => AuroraCtrlxD,
                    AuroraStatusxDO           => AuroraStatusxD,
                    -- DRP Port
                    AuroraDRPM2SxDI           => AuroraDRPM2SxD,
                    AuroraDRPS2MxDO           => AuroraDRPS2MxD,
                    -- RX Fifo Status
                    -- North
                    NorthRXFifoStatusxDO      => NorthRXFifoStatusxD,
                    -- East
                    EastRXFifoStatusxDO       => EastRXFifoStatusxD,
                    -- South
                    SouthRXFifoStatusxDO      => SouthRXFifoStatusxD,
                    -- West
                    WestRXFifoStatusxDO       => WestRXFifoStatusxD,
                    -- Axis Fifo Error
                    AxisFifoErrorxDO          => AxisFifoErrorxD);

        end block GTPhyxB;

        NetworkLayerxB : block is

            constant C_SCALP_PACKET_PAYLOAD_SIZE : integer range 1 to C_SCALP_PACKET_LENGTH_RANGE_VALUE := 8;
            constant C_SCALP_RANDOM_READY        : boolean                                              := false;
            ---------------------------------------------------------------------------
            -- Scalp Packets
            ---------------------------------------------------------------------------

            constant C_SCALP_PACKET_NET_ADDR_110 : t_scalp_netaddr
                := (XxD => 1, YxD => 1, ZxD => 0);
            constant C_SCALP_PACKET_NET_ADDR_210 : t_scalp_netaddr
                := (XxD => 2, YxD => 1, ZxD => 0);
            ---------------------------------------------------------------------------
            -- Scalp Packet Headers
            ---------------------------------------------------------------------------
            constant C_SP_HEADER_NULL       : t_scalp_packet_header := C_NO_SCALP_PACKET_HEADER;
            constant C_SP_HEADER_110_TO_210 : t_scalp_packet_header
                := (DstAddrxD => C_SCALP_PACKET_NET_ADDR_210,
                    SrcAddrxD => C_SCALP_PACKET_NET_ADDR_110,
                    TypexD    => 1,
                    LengthxD  => C_SCALP_PACKET_PAYLOAD_SIZE);
            ---------------------------------------------------------------------------
            -- Scalp Packet Payloads
            ---------------------------------------------------------------------------
            constant C_SP_PAYLOAD_NULL : t_scalp_packet_payload(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))
                := (0 => C_NO_SCALP_PACKET_WORD,
                    1 => C_NO_SCALP_PACKET_WORD,
                    2 => C_NO_SCALP_PACKET_WORD,
                    3 => C_NO_SCALP_PACKET_WORD,
                    4 => C_NO_SCALP_PACKET_WORD,
                    5 => C_NO_SCALP_PACKET_WORD,
                    6 => C_NO_SCALP_PACKET_WORD,
                    7 => C_NO_SCALP_PACKET_WORD);
            constant C_SP_PAYLOAD_0 : t_scalp_packet_payload(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))
                := (0 =>
                    (WordxD => std_ulogic_vector(to_unsigned(16#0abbccdd#, (C_BYTE_SIZE * 4))),
                     IdxD   => 0),
                    1 =>
                    (WordxD => std_ulogic_vector(to_unsigned(16#0bccddee#, (C_BYTE_SIZE * 4))),
                     IdxD   => 1),
                    2 =>
                    (WordxD => std_ulogic_vector(to_unsigned(16#0cddeeff#, (C_BYTE_SIZE * 4))),
                     IdxD   => 2),
                    3 =>
                    (WordxD => std_ulogic_vector(to_unsigned(16#0deeff11#, (C_BYTE_SIZE * 4))),
                     IdxD   => 3),
                    4 =>
                    (WordxD => std_ulogic_vector(to_unsigned(16#0eff1122#, (C_BYTE_SIZE * 4))),
                     IdxD   => 4),
                    5 =>
                    (WordxD => std_ulogic_vector(to_unsigned(16#0f112233#, (C_BYTE_SIZE * 4))),
                     IdxD   => 5),
                    6 =>
                    (WordxD => std_ulogic_vector(to_unsigned(16#01223344#, (C_BYTE_SIZE * 4))),
                     IdxD   => 6),
                    7 =>
                    (WordxD => std_ulogic_vector(to_unsigned(16#02334455#, (C_BYTE_SIZE * 4))),
                     IdxD   => 7));

            type t_write_sp_states is (E_WR_SP_IDLE, E_WR_SP_VALID, E_WR_SP_WAIT);

            signal ScalpRouterResetxRNA : std_ulogic := '0';
            -- Scalp Packets
            -- From South 101
            ---------------------------------------------------------------------------
            ---------------------------------------------------------------------------
            signal ScalpPacketLocalxD   : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
                (SpHeaderxD  => C_SP_HEADER_NULL,
                 SpPayloadxD => C_SP_PAYLOAD_NULL);
            signal ScalpPacketValidLocalxS  : std_ulogic := '0';
            signal ScalpPacketSelectLocalxD : integer    := 0;
            ---------------------------------------------------------------------------
            -- Packet 0
            signal ScalpPacket0xD           : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
                (SpHeaderxD  => C_SP_HEADER_110_TO_210,
                 SpPayloadxD => C_SP_PAYLOAD_0);
            signal ScalpPacketValid12xS : std_ulogic        := '0';
            --
            signal WrSPStatexDP         : t_write_sp_states := E_WR_SP_IDLE;
            signal WrSPStatexDN         : t_write_sp_states := E_WR_SP_IDLE;
            signal VioWrSpValidxS       : std_ulogic        := '0';

            -- attribute mark_debug of ScalpRouterResetxRNA : signal is "true";
            -- attribute keep of ScalpRouterResetxRNA       : signal is "true";
            -- attribute mark_debug of ScalpPacket0xD       : signal is "true";
            -- attribute keep of ScalpPacket0xD             : signal is "true";
            -- attribute mark_debug of WrSPStatexDP         : signal is "true";
            -- attribute keep of WrSPStatexDP               : signal is "true";
            -- attribute mark_debug of WrSPStatexDN         : signal is "true";
            -- attribute keep of WrSPStatexDN               : signal is "true";
            -- attribute mark_debug of VioWrSpValidxS       : signal is "true";
            -- attribute keep of VioWrSpValidxS             : signal is "true";
            -- attribute mark_debug of ScalpPacketLocalxD      : signal is "true";
            -- attribute keep of ScalpPacketLocalxD            : signal is "true";
            -- attribute mark_debug of ScalpPacketValidLocalxS : signal is "true";
            -- attribute keep of ScalpPacketValidLocalxS       : signal is "true";

        begin  -- block NetworkLayerxB

            ResetxB : block is

                constant C_CDC_TYPE     : integer range 0 to 2  := 1;
                constant C_RESET_STATE  : integer range 0 to 1  := 0;
                constant C_SINGLE_BIT   : integer range 0 to 1  := 1;
                constant C_FLOP_INPUT   : integer range 0 to 1  := 1;
                constant C_VECTOR_WIDTH : integer range 0 to 32 := 2;
                constant C_MTBF_STAGES  : integer range 0 to 6  := 5;

                signal PrimaryResetxRN   : std_ulogic := '0';
                signal SecondaryResetxRN : std_ulogic := '0';
                signal PSSysResetSyncxR  : std_ulogic := '0';

            begin  -- block ResetxB

                PrimaryResetxAS   : PrimaryResetxRN   <= not PSSysResetxR;
                SecondaryResetxAS : SecondaryResetxRN <= not AuroraClkMasterxC.PllNotLockedxS;

                CDCSyncResetxB : entity work.cdc_sync
                    generic map (
                        C_CDC_TYPE     => C_CDC_TYPE,
                        C_RESET_STATE  => C_RESET_STATE,
                        C_SINGLE_BIT   => C_SINGLE_BIT,
                        C_FLOP_INPUT   => C_FLOP_INPUT,
                        C_VECTOR_WIDTH => C_VECTOR_WIDTH,
                        C_MTBF_STAGES  => C_MTBF_STAGES)
                    port map (
                        PrimaryClkxCAI     => PSSysClkxC,
                        PrimaryResetxRNI   => PrimaryResetxRN,
                        PrimaryxSI         => PSSysResetxR,
                        PrimaryxDI         => (others => '0'),
                        PrimaryAckxSO      => open,
                        SecondaryClkxCAI   => AuroraClkMasterxC.UserClkxC,
                        SecondaryResetxRNI => SecondaryResetxRN,
                        SecondaryxSO       => PSSysResetSyncxR,
                        SecondaryxDO       => open);

                ScalpRouterResetxAS : ScalpRouterResetxRNA <= (not PSSysResetSyncxR) and
                                                              (not AuroraClkMasterxC.PllNotLockedxS);
                -- RX Fifo reset 196 cycles
                -- Clock and Resets
                -- RX Fifo
                NorthFifoResetxAS : RXResetxR.FifoResetxR.NorthxR <=
                    '1' when
                    (AuroraStatusxD.LaneUpxD(0)(0) = '0') or
                    (AuroraStatusxD.ChannelUpxD(0) = '0') else
                    '0';
                EastFifoResetxAS : RXResetxR.FifoResetxR.EastxR <=
                    '1' when
                    (AuroraStatusxD.LaneUpxD(1)(0) = '0') or
                    (AuroraStatusxD.ChannelUpxD(1) = '0') else
                    '0';
                SouthFifoResetxAS : RXResetxR.FifoResetxR.SouthxR <=
                    '1' when
                    (AuroraStatusxD.LaneUpxD(2)(0) = '0') or
                    (AuroraStatusxD.ChannelUpxD(2) = '0') else
                    '0';
                WestFifoResetxAS : RXResetxR.FifoResetxR.WestxR <=
                    '1' when
                    (AuroraStatusxD.LaneUpxD(3)(0) = '0') or
                    (AuroraStatusxD.ChannelUpxD(3) = '0') else
                    '0';
                -- Back pressure
                NorthBackPressureResetxAS : RXResetxR.BackPressureResetxR.NorthxR <=
                    '1' when
                    (AuroraStatusxD.LaneUpxD(0)(0) = '0') or
                    (RXFifoResetDonexD.NorthxS = '0') or
                    (RXFifoResetDoneDelayedxD.NorthxS = '0') or
                    (AuroraStatusxD.ChannelUpxD(0) = '0') else
                    '0';
                EastBackPressureResetxAS : RXResetxR.BackPressureResetxR.EastxR <=
                    '1' when
                    (AuroraStatusxD.LaneUpxD(1)(0) = '0') or
                    (RXFifoResetDonexD.EastxS = '0') or
                    (RXFifoResetDoneDelayedxD.EastxS = '0') or
                    (AuroraStatusxD.ChannelUpxD(1) = '0') else
                    '0';
                SouthBackPressureResetxAS : RXResetxR.BackPressureResetxR.SouthxR <=
                    '1' when
                    (RXFifoResetDonexD.SouthxS = '0') or
                    (RXFifoResetDoneDelayedxD.SouthxS = '0') or
                    (AuroraStatusxD.LaneUpxD(2)(0) = '0') or
                    (AuroraStatusxD.ChannelUpxD(2) = '0') else
                    '0';
                WestBackPressureResetxAS : RXResetxR.BackPressureResetxR.WestxR <=
                    '1' when
                    (AuroraStatusxD.LaneUpxD(3)(0) = '0') or
                    (RXFifoResetDonexD.WestxS = '0') or
                    (RXFifoResetDoneDelayedxD.WestxS = '0') or
                    (AuroraStatusxD.ChannelUpxD(3) = '0') else
                    '0';

            end block ResetxB;

            -- ScalpRouterReadyxB : block is
            -- begin  -- block ScalpRouterReadyxB

            --     ScalpRouterReadyNorthxAS : ScalpRouterReadyxD.NorthxS <=
            --         '0' when
            --         (RXFifoResetDonexD.NorthxS = '0') or
            --         (RXFifoResetDoneDelayedxD.NorthxS = '0') or
            --         (AuroraStatusxD.LaneUpxD(0)(0) = '0') or
            --         (AuroraStatusxD.ChannelUpxD(0) = '0') else
            --         '1';
            --     ScalpRouterReadyEastxAS : ScalpRouterReadyxD.EastxS <=
            --         '0' when
            --         (RXFifoResetDonexD.EastxS = '0') or
            --         (RXFifoResetDoneDelayedxD.EastxS = '0') or
            --         (AuroraStatusxD.LaneUpxD(1)(0) = '0') or
            --         (AuroraStatusxD.ChannelUpxD(1) = '0') else
            --         '1';
            --     ScalpRouterReadySouthxAS : ScalpRouterReadyxD.SouthxS <=
            --         '0' when
            --         (RXFifoResetDonexD.SouthxS = '0') or
            --         (RXFifoResetDoneDelayedxD.SouthxS = '0') or
            --         (AuroraStatusxD.LaneUpxD(2)(0) = '0') or
            --         (AuroraStatusxD.ChannelUpxD(2) = '0') else
            --         '1';
            --     ScalpRouterReadyWestxAS : ScalpRouterReadyxD.WestxS <=
            --         '0' when
            --         (RXFifoResetDonexD.WestxS = '0') or
            --         (RXFifoResetDoneDelayedxD.WestxS = '0') or
            --         (AuroraStatusxD.LaneUpxD(3)(0) = '0') or
            --         (AuroraStatusxD.ChannelUpxD(3) = '0') else
            --         '1';
            --     ScalpRouterReadyTopxAS    : ScalpRouterReadyxD.TopxS    <= '0';
            --     ScalpRouterReadyBottomxAS : ScalpRouterReadyxD.BottomxS <= '0';
            --     ScalpRouterReadyLocalxAS  : ScalpRouterReadyxD.LocalxS  <= (not PSSysResetxR) and
            --                                                              (not AuroraClkMasterxC.PllNotLockedxS);

            -- end block ScalpRouterReadyxB;

            NorthTXM2SxAS : NorthTXM2SxD                     <= TXAxism2sVectorxD(C_NORTH_IF_ID);
            EastTXM2SxAS  : EastTXM2SxD                      <= TXAxism2sVectorxD(C_EAST_IF_ID);
            SouthTXM2SxAS : SouthTXM2SxD                     <= TXAxism2sVectorxD(C_SOUTH_IF_ID);
            WestTXM2SxAS  : WestTXM2SxD                      <= TXAxism2sVectorxD(C_WEST_IF_ID);
            NorthTXS2MxAS : TXAxiss2mVectorxD(C_NORTH_IF_ID) <= NorthTXS2MxD;
            EastTXS2MxAS  : TXAxiss2mVectorxD(C_EAST_IF_ID)  <= EastTXS2MxD;
            SouthTXS2MxAS : TXAxiss2mVectorxD(C_SOUTH_IF_ID) <= SouthTXS2MxD;
            WestTXS2MxAS  : TXAxiss2mVectorxD(C_WEST_IF_ID)  <= WestTXS2MxD;
            NorthRXM2SxAS : RXAxism2sVectorxD(C_NORTH_IF_ID) <= NorthRXM2SxD;
            EastRXM2SxAS  : RXAxism2sVectorxD(C_EAST_IF_ID)  <= EastRXM2SxD;
            SouthRXM2SxAS : RXAxism2sVectorxD(C_SOUTH_IF_ID) <= SouthRXM2SxD;
            WestRXM2SxAS  : RXAxism2sVectorxD(C_WEST_IF_ID)  <= WestRXM2SxD;
            NorthRXS2MxAS : NorthRXS2MxD                     <= RXAxiss2mVectorxD(C_NORTH_IF_ID);
            EastRXS2MxAS  : EastRXS2MxD                      <= RXAxiss2mVectorxD(C_EAST_IF_ID);
            SouthRXS2MxAS : SouthRXS2MxD                     <= RXAxiss2mVectorxD(C_SOUTH_IF_ID);
            WestRXS2MxAS  : WestRXS2MxD                      <= RXAxiss2mVectorxD(C_WEST_IF_ID);
            -- Local Router Net Addr
            LocNetAddrxAS : LocNetAddrxD                     <= C_SCALP_PACKET_NET_ADDR_110;
            -- Local Ready
            -- RXAxiss2mVectorxD(C_LOCAL_IF_ID) <= 

            WrSpValidxI : entity work.vio_axi_cnt_ctrl
                port map (
                    clk           => AuroraClkMasterxC.UserClkxC,
                    probe_out0(0) => VioWrSpValidxS);

            ScalpSP2AxisLocalxI : entity work.scalp_sp_to_axis
                generic map (
                    C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE)
                port map (
                    SysClkxCI           => AuroraClkMasterxC.UserClkxC,
                    SysRstxRNAI         => ScalpRouterResetxRNA,
                    ScalpPacketxDI      => ScalpPacket0xD,
                    ScalpPacketValidxSI => ScalpPacketValid12xS,
                    ScalpAxism2sxDO     => RXAxism2sVectorxD(C_LOCAL_IF_ID),
                    ScalpAxiss2mxDI     => RXAxiss2mVectorxD(C_LOCAL_IF_ID),
                    ScalpRdyxSO         => open);

            ScalpRouterxI : entity work.scalp_router
                generic map (
                    C_SCALP_NUMBER_OF_INTERFACE => C_SCALP_NUMBER_OF_INTERFACE,
                    C_SCALP_SCHEDULER_STRATEGY  => C_SCALP_SCHEDULER_STRATEGY)
                port map (
                    SysClkxCI          => AuroraClkMasterxC.UserClkxC,
                    SysRstxRNAI        => ScalpRouterResetxRNA,
                    LocNetAddrxDI      => LocNetAddrxD,
                    RXAxism2sVectorxDI => RXAxism2sVectorxD,
                    RXAxiss2mVectorxDO => RXAxiss2mVectorxD,
                    TXAxism2sVectorxDO => TXAxism2sVectorxD,
                    TXAxiss2mVectorxDI => TXAxiss2mVectorxD,
                    QoSVectorxDI       => QoSVectorxD);

            ScalpAxis2SPxI : entity work.scalp_axis_to_sp
                generic map (
                    C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE,
                    C_SCALP_RANDOM_READY        => C_SCALP_RANDOM_READY)
                port map (
                    SysClkxCI           => AuroraClkMasterxC.UserClkxC,
                    SysRstxRNAI         => ScalpRouterResetxRNA,
                    ScalpAxism2sxDI     => TXAxism2sVectorxD(C_LOCAL_IF_ID),
                    ScalpAxiss2mxDO     => TXAxiss2mVectorxD(C_LOCAL_IF_ID),
                    ScalpPacketxDO      => ScalpPacketLocalxD,
                    ScalpPacketValidxSO => ScalpPacketValidLocalxS);

            WritePacketxB : block is
            begin  -- block WritePacketxB

                UpdateRegxP : process (AuroraClkMasterxC.UserClkxC,
                                       ScalpRouterResetxRNA) is
                begin  -- process UpdateRegxP
                    if ScalpRouterResetxRNA = '0' then
                        WrSPStatexDP <= E_WR_SP_IDLE;
                    elsif rising_edge(AuroraClkMasterxC.UserClkxC) then
                        WrSPStatexDP <= WrSPStatexDN;
                    end if;
                end process UpdateRegxP;

                SpValidxP : process (VioWrSpValidxS, WrSPStatexDP) is
                begin  -- process SpValidxP
                    -- Default values
                    WrSPStatexDN         <= WrSPStatexDP;
                    ScalpPacketValid12xS <= '0';

                    case WrSPStatexDP is
                        when E_WR_SP_IDLE =>
                                if VioWrSpValidxS = '1' then
                                    ScalpPacketValid12xS <= '1';
                                    WrSPStatexDN         <= E_WR_SP_VALID;
                                end if;
                        when E_WR_SP_VALID =>
                                ScalpPacketValid12xS <= '0';
                                WrSPStatexDN         <= E_WR_SP_WAIT;
                        when E_WR_SP_WAIT =>
                                if VioWrSpValidxS = '0' then
                                    WrSPStatexDN <= E_WR_SP_IDLE;
                                end if;
                        when others => null;
                    end case;
                end process SpValidxP;

            end block WritePacketxB;

        end block NetworkLayerxB;

        ZynqRegBankxB : block is
        begin  -- block ZynqRegBankxB

            RegBankxB : block is
            begin  -- block RegBankxB

                WriteRegPortxP : process (NorthCtrlRegPortxDP, WrAddrxD,
                                          WrDataxD, WrValidxS) is
                begin  -- process WriteRegPortxP
                    -- North                    
                    NorthCtrlRegPortxDN <= NorthCtrlRegPortxDP;
                    -- NorthWrDataRegPortxDN <= NorthWrDataRegPortxDP;
                    -- -- East
                    -- EastCtrlRegPortxDN    <= EastCtrlRegPortxDP;
                    -- EastWrDataRegPortxDN  <= EastWrDataRegPortxDP;
                    -- -- South
                    -- SouthCtrlRegPortxDN   <= SouthCtrlRegPortxDP;
                    -- SouthWrDataRegPortxDN <= SouthWrDataRegPortxDP;
                    -- -- West
                    -- WestCtrlRegPortxDN    <= WestCtrlRegPortxDP;
                    -- WestWrDataRegPortxDN  <= WestWrDataRegPortxDP;

                    if WrValidxS = '1' then
                        case WrAddrxD is
                            -- Ctrl
                            -- North
                            when x"000" => NorthCtrlRegPortxDN <= WrDataxD;
                            when x"004" => NorthCtrlRegPortxDN <= NorthCtrlRegPortxDP or WrDataxD;
                            when x"008" => NorthCtrlRegPortxDN <= NorthCtrlRegPortxDP and not WrDataxD;
                            -- East
                            -- when x"00c" => EastCtrlRegPortxDN    <= WrDataxD;
                            -- when x"010" => EastCtrlRegPortxDN    <= EastCtrlRegPortxDP or WrDataxD;
                            -- when x"014" => EastCtrlRegPortxDN    <= EastCtrlRegPortxDP and not WrDataxD;
                            -- -- South
                            -- when x"018" => SouthCtrlRegPortxDN   <= WrDataxD;
                            -- when x"01c" => SouthCtrlRegPortxDN   <= SouthCtrlRegPortxDP or WrDataxD;
                            -- when x"020" => SouthCtrlRegPortxDN   <= SouthCtrlRegPortxDP and not WrDataxD;
                            -- -- East
                            -- when x"024" => WestCtrlRegPortxDN    <= WrDataxD;
                            -- when x"028" => WestCtrlRegPortxDN    <= WestCtrlRegPortxDP or WrDataxD;
                            -- when x"02c" => WestCtrlRegPortxDN    <= WestCtrlRegPortxDP and not WrDataxD;
                            -- -- Data
                            -- -- North
                            -- when x"030" => NorthWrDataRegPortxDN <= WrDataxD;
                            -- -- East
                            -- when x"034" => EastWrDataRegPortxDN  <= WrDataxD;
                            -- -- South
                            -- when x"038" => SouthWrDataRegPortxDN <= WrDataxD;
                            -- -- West
                            -- when x"03c" => WestWrDataRegPortxDN  <= WrDataxD;
                            when others => null;
                        end case;
                    end if;
                end process WriteRegPortxP;

                ReadRegPortxP : process (PSSysClkxC, PSSysResetxR) is
                begin  -- process ReadRegPortxP
                    if PSSysResetxR = '1' then
                        RdDataxD <= (others => '0');
                    elsif rising_edge(PSSysClkxC) then
                        RdDataxD <= (others => '0');

                        if RdValidxS = '1' then
                            case RdAddrxD is
                                when x"000" => RdDataxD <= NorthCtrlRegPortxDP;
                                -- when x"00C" => RdDataxD <= EastCtrlRegPortxDP;
                                -- when x"018" => RdDataxD <= SouthCtrlRegPortxDP;
                                -- when x"024" => RdDataxD <= WestCtrlRegPortxDP;
                                -- when x"030" => RdDataxD <= NorthWrDataRegPortxDP;
                                -- when x"034" => RdDataxD <= EastWrDataRegPortxDP;
                                -- when x"038" => RdDataxD <= SouthWrDataRegPortxDP;
                                -- when x"03c" => RdDataxD <= WestWrDataRegPortxDP;
                                -- when x"040" => RdDataxD <= NorthStatusRegPortxDP;
                                -- when x"044" => RdDataxD <= EastStatusRegPortxDP;
                                -- when x"048" => RdDataxD <= SouthStatusRegPortxDP;
                                -- when x"04c" => RdDataxD <= WestStatusRegPortxDP;
                                when others => RdDataxD <= x"aabbccdd";
                            end case;
                        end if;
                    end if;
                end process ReadRegPortxP;

                RegBankxP : process (PSSysClkxC, PSSysResetxR) is
                begin  -- process RegBankxP
                    if PSSysResetxR = '1' then
                        -- North
                        -- NorthStatusRegPortxDP <= (others => '0');
                        NorthCtrlRegPortxDP <= (others => '0');
                    -- NorthWrDataRegPortxDP <= (others => '0');
                    -- East
                    -- EastStatusRegPortxDP  <= (others => '0');
                    -- EastCtrlRegPortxDP    <= (others => '0');
                    -- EastWrDataRegPortxDP  <= (others => '0');
                    -- -- South
                    -- SouthStatusRegPortxDP <= (others => '0');
                    -- SouthCtrlRegPortxDP   <= (others => '0');
                    -- SouthWrDataRegPortxDP <= (others => '0');
                    -- -- West
                    -- WestStatusRegPortxDP  <= (others => '0');
                    -- WestCtrlRegPortxDP    <= (others => '0');
                    -- WestWrDataRegPortxDP  <= (others => '0');
                    elsif rising_edge(PSSysClkxC) then
                        -- North
                        -- NorthStatusRegPortxDP <= NorthStatusRegPortxDN;
                        NorthCtrlRegPortxDP <= NorthCtrlRegPortxDN;
                    -- NorthWrDataRegPortxDP <= NorthWrDataRegPortxDN;
                    -- -- East
                    -- EastStatusRegPortxDP  <= EastStatusRegPortxDN;
                    -- EastCtrlRegPortxDP    <= EastCtrlRegPortxDN;
                    -- EastWrDataRegPortxDP  <= EastWrDataRegPortxDN;
                    -- -- South
                    -- SouthStatusRegPortxDP <= SouthStatusRegPortxDN;
                    -- SouthCtrlRegPortxDP   <= SouthCtrlRegPortxDN;
                    -- SouthWrDataRegPortxDP <= SouthWrDataRegPortxDN;
                    -- -- West
                    -- WestStatusRegPortxDP  <= WestStatusRegPortxDN;
                    -- WestCtrlRegPortxDP    <= WestCtrlRegPortxDN;
                    -- WestWrDataRegPortxDP  <= WestWrDataRegPortxDN;
                    end if;
                end process RegBankxP;

            end block RegBankxB;

            --     TxFifoxB : block is
            --     begin  -- block TxFifoxB

            --         NorthWrDataxAS   : NorthNativeSlavexD.DataxD <= NorthWrDataRegPortxDP;
            --         EastWrDataxAS    : EastNativeSlavexD.DataxD  <= EastWrDataRegPortxDP;
            --         SouthWrDataxAS   : SouthNativeSlavexD.DataxD <= SouthWrDataRegPortxDN;
            --         WestWrDataRegxAS : WestNativeSlavexD.DataxD  <= WestWrDataRegPortxDN;
            --         NorthWrEnxAS     : NorthNativeSlavexD.WrEnxS <= NorthCtrlRegPortxDP(0);
            --         EastWrEnxAS      : EastNativeSlavexD.WrEnxS  <= EastCtrlRegPortxDP(0);
            --         SouthWrEnxAS     : SouthNativeSlavexD.WrEnxS <= SouthCtrlRegPortxDP(0);
            --         WestWrEnxAS      : EastNativeSlavexD.WrEnxS  <= EastCtrlRegPortxDP(0);
            --         NorthStatusRegPortxAS : NorthStatusRegPortxDN <= (0      => NorthNativeMasterxD.FullxS,
            --                                                           1      => NorthNativeMasterxD.EmptyxS,
            --                                                           2      => NorthNativeMasterxD.AlmostFullxS,
            --                                                           3      => NorthNativeMasterxD.AlmostEmptyxS,
            --                                                           4      => NorthNativeMasterxD.WrRstBusyxS,
            --                                                           5      => NorthNativeMasterxD.RdRstBusyxS,
            --                                                           others => '0');
            --         EastStatusRegPortxAS : EastStatusRegPortxDN <= (0      => EastNativeMasterxD.FullxS,
            --                                                         1      => EastNativeMasterxD.EmptyxS,
            --                                                         2      => EastNativeMasterxD.AlmostFullxS,
            --                                                         3      => EastNativeMasterxD.AlmostEmptyxS,
            --                                                         4      => EastNativeMasterxD.WrRstBusyxS,
            --                                                         5      => EastNativeMasterxD.RdRstBusyxS,
            --                                                         others => '0');
            --         SouthStatusRegPortxAS : SouthStatusRegPortxDN <= (0      => SouthNativeMasterxD.FullxS,
            --                                                           1      => SouthNativeMasterxD.EmptyxS,
            --                                                           2      => SouthNativeMasterxD.AlmostFullxS,
            --                                                           3      => SouthNativeMasterxD.AlmostEmptyxS,
            --                                                           4      => SouthNativeMasterxD.WrRstBusyxS,
            --                                                           5      => SouthNativeMasterxD.RdRstBusyxS,
            --                                                           others => '0');
            --         WestStatusRegPortxAS : WestStatusRegPortxDN <= (0      => WestNativeMasterxD.FullxS,
            --                                                         1      => WestNativeMasterxD.EmptyxS,
            --                                                         2      => WestNativeMasterxD.AlmostFullxS,
            --                                                         3      => WestNativeMasterxD.AlmostEmptyxS,
            --                                                         4      => WestNativeMasterxD.WrRstBusyxS,
            --                                                         5      => WestNativeMasterxD.RdRstBusyxS,
            --                                                         others => '0');

            --         UpdateRegxP : process (AuroraClkMasterxC.PllNotLockedxS,
            --                                AuroraClkMasterxC.UserClkxC) is
            --         begin  -- process UpdateRegxP
            --             if not AuroraClkMasterxC.PllNotLockedxS then
            --                 NorthStatusSendWordxDP <= E_IDLE;
            --                 EastStatusSendWordxDP  <= E_IDLE;
            --                 SouthStatusSendWordxDP <= E_IDLE;
            --                 WestStatusSendWordxDP  <= E_IDLE;
            --             elsif rising_edge(AuroraClkMasterxC.UserClkxC) then
            --                 NorthStatusSendWordxDP <= NorthStatusSendWordxDN;
            --                 EastStatusSendWordxDP  <= EastStatusSendWordxDN;
            --                 SouthStatusSendWordxDP <= SouthStatusSendWordxDN;
            --                 WestStatusSendWordxDP  <= WestStatusSendWordxDN;
            --             end if;
            --         end process UpdateRegxP;

            --         NorthSendWordxP : process (NorthNativeMasterxD.DataxD,
            --                                    NorthNativeMasterxD.EmptyxS,
            --                                    NorthStatusSendWordxDP,
            --                                    NorthTXS2MxD.ReadyxS) is
            --         begin  -- process NorthSendWordxP
            --             NorthTXM2SxD.DataxD       <= (others => '0');
            --             NorthTXM2SxD.KeepxD       <= (others => '1');
            --             NorthTXM2SxD.LastxS       <= '0';
            --             NorthTXM2SxD.ValidxS      <= '0';
            --             NorthNativeSlavexD.RdEnxS <= '0';
            --             NorthStatusSendWordxDN    <= NorthStatusSendWordxDP;

            --             case NorthStatusSendWordxDP is
            --                 when E_IDLE =>

            --                         if (NorthNativeMasterxD.EmptyxS = '0') and
            --                             (NorthTXS2MxD.ReadyxS = '1') then
            --                             NorthTXM2SxD.DataxD       <= NorthNativeMasterxD.DataxD;
            --                             NorthTXM2SxD.LastxS       <= '1';
            --                             NorthTXM2SxD.ValidxS      <= '1';
            --                             NorthNativeSlavexD.RdEnxS <= '1';
            --                             NorthStatusSendWordxDN    <= E_SEND;
            --                         end if;

            --                 when E_SEND =>
            --                         NorthStatusSendWordxDN <= E_IDLE;

            --                 when others => null;
            --             end case;
            --         end process NorthSendWordxP;

            --         EastSendWordxP : process (EastNativeMasterxD.DataxD,
            --                                   EastNativeMasterxD.EmptyxS,
            --                                   EastStatusSendWordxDP,
            --                                   EastTXS2MxD.ReadyxS) is
            --         begin  -- process EastSendWordxP
            --             EastTXM2SxD.DataxD       <= (others => '0');
            --             EastTXM2SxD.KeepxD       <= (others => '1');
            --             EastTXM2SxD.LastxS       <= '0';
            --             EastTXM2SxD.ValidxS      <= '0';
            --             EastNativeSlavexD.RdEnxS <= '0';
            --             EastStatusSendWordxDN    <= EastStatusSendWordxDP;

            --             case EastStatusSendWordxDP is
            --                 when E_IDLE =>

            --                         if (EastNativeMasterxD.EmptyxS = '0') and
            --                             (EastTXS2MxD.ReadyxS = '1') then
            --                             EastTXM2SxD.DataxD       <= EastNativeMasterxD.DataxD;
            --                             EastTXM2SxD.LastxS       <= '1';
            --                             EastTXM2SxD.ValidxS      <= '1';
            --                             EastNativeSlavexD.RdEnxS <= '1';
            --                             EastStatusSendWordxDN    <= E_SEND;
            --                         end if;

            --                 when E_SEND =>
            --                         EastStatusSendWordxDN <= E_IDLE;

            --                 when others => null;
            --             end case;
            --         end process EastSendWordxP;

            --         SouthSendWordxP : process (SouthNativeMasterxD.DataxD,
            --                                    SouthNativeMasterxD.EmptyxS,
            --                                    SouthStatusSendWordxDP,
            --                                    SouthTXS2MxD.ReadyxS) is
            --         begin  -- process SouthSendWordxP
            --             SouthTXM2SxD.DataxD       <= (others => '0');
            --             SouthTXM2SxD.KeepxD       <= (others => '1');
            --             SouthTXM2SxD.LastxS       <= '0';
            --             SouthTXM2SxD.ValidxS      <= '0';
            --             SouthNativeSlavexD.RdEnxS <= '0';
            --             SouthStatusSendWordxDN    <= SouthStatusSendWordxDP;

            --             case SouthStatusSendWordxDP is
            --                 when E_IDLE =>

            --                         if (SouthNativeMasterxD.EmptyxS = '0') and
            --                             (SouthTXS2MxD.ReadyxS = '1') then
            --                             SouthTXM2SxD.DataxD       <= SouthNativeMasterxD.DataxD;
            --                             SouthTXM2SxD.LastxS       <= '1';
            --                             SouthTXM2SxD.ValidxS      <= '1';
            --                             SouthNativeSlavexD.RdEnxS <= '1';
            --                             SouthStatusSendWordxDN    <= E_SEND;
            --                         end if;

            --                 when E_SEND =>
            --                         SouthStatusSendWordxDN <= E_IDLE;

            --                 when others => null;
            --             end case;
            --         end process SouthSendWordxP;

            --         WestSendWordxP : process (WestNativeMasterxD.DataxD,
            --                                   WestNativeMasterxD.EmptyxS,
            --                                   WestStatusSendWordxDP,
            --                                   WestTXS2MxD.ReadyxS) is
            --         begin  -- process WestSendWordxP
            --             WestTXM2SxD.DataxD       <= (others => '0');
            --             WestTXM2SxD.KeepxD       <= (others => '1');
            --             WestTXM2SxD.LastxS       <= '0';
            --             WestTXM2SxD.ValidxS      <= '0';
            --             WestNativeSlavexD.RdEnxS <= '0';
            --             WestStatusSendWordxDN    <= WestStatusSendWordxDP;

            --             case WestStatusSendWordxDP is
            --                 when E_IDLE =>

            --                         if (WestNativeMasterxD.EmptyxS = '0') and
            --                             (WestTXS2MxD.ReadyxS = '1') then
            --                             WestTXM2SxD.DataxD       <= WestNativeMasterxD.DataxD;
            --                             WestTXM2SxD.LastxS       <= '1';
            --                             WestTXM2SxD.ValidxS      <= '1';
            --                             WestNativeSlavexD.RdEnxS <= '1';
            --                             WestStatusSendWordxDN    <= E_SEND;
            --                         end if;

            --                 when E_SEND =>
            --                         WestStatusSendWordxDN <= E_IDLE;

            --                 when others => null;
            --             end case;
            --         end process WestSendWordxP;

            --         NorthFifoxI : entity work.scalp_packet_fifo_wrapper
            --             port map (
            --                 RdClkxCI        => AuroraClkMasterxC.UserClkxC,
            --                 WrClkxCI        => PSSysClkxC,
            --                 ResetxRI        => PSSysResetxR,
            --                 NativeSlavexDI  => NorthNativeSlavexD,
            --                 NativeMasterxDO => NorthNativeMasterxD);

            --         EastFifoxI : entity work.scalp_packet_fifo_wrapper
            --             port map (
            --                 RdClkxCI        => AuroraClkMasterxC.UserClkxC,
            --                 WrClkxCI        => PSSysClkxC,
            --                 ResetxRI        => PSSysResetxR,
            --                 NativeSlavexDI  => EastNativeSlavexD,
            --                 NativeMasterxDO => EastNativeMasterxD);

            --         SouthFifoxI : entity work.scalp_packet_fifo_wrapper
            --             port map (
            --                 RdClkxCI        => AuroraClkMasterxC.UserClkxC,
            --                 WrClkxCI        => PSSysClkxC,
            --                 ResetxRI        => PSSysResetxR,
            --                 NativeSlavexDI  => SouthNativeSlavexD,
            --                 NativeMasterxDO => SouthNativeMasterxD);

            --         WestFifoxI : entity work.scalp_packet_fifo_wrapper
            --             port map (
            --                 RdClkxCI        => AuroraClkMasterxC.UserClkxC,
            --                 WrClkxCI        => PSSysClkxC,
            --                 ResetxRI        => PSSysResetxR,
            --                 NativeSlavexDI  => WestNativeSlavexD,
            --                 NativeMasterxDO => WestNativeMasterxD);

            --     end block TxFifoxB;

        end block ZynqRegBankxB;

        -- DebugxB : block is

        --     -- RX Fifo reset 196 cycles
        --     constant C_RX_FIFO_RST_DLY_TICKS : integer := 200;

        --     type t_read_phy_states is (E_READ_PHY_IDLE, E_READ_PHY_S0, E_READ_PHY_S1, E_READ_PHY_S2);
        --     type t_write_phy_states is (E_WRITE_PHY_IDLE, E_WRITE_PHY_W0, E_WRITE_PHY_W1,
        --                                 E_WRITE_PHY_W2, E_WRITE_PHY_W3, E_WRITE_PHY_W4,
        --                                 E_WRITE_PHY_W5, E_WRITE_PHY_W6, E_WRITE_PHY_W7);

        --     -- North
        --     signal NorthReadPhyStatexDN  : t_read_phy_states  := E_READ_PHY_IDLE;
        --     signal NorthReadPhyStatexDP  : t_read_phy_states  := E_READ_PHY_IDLE;
        --     signal NorthWritePhyStatexDN : t_write_phy_states := E_WRITE_PHY_IDLE;
        --     signal NorthWritePhyStatexDP : t_write_phy_states := E_WRITE_PHY_IDLE;
        --     -- East
        --     signal EastReadPhyStatexDN   : t_read_phy_states  := E_READ_PHY_IDLE;
        --     signal EastReadPhyStatexDP   : t_read_phy_states  := E_READ_PHY_IDLE;
        --     signal EastWritePhyStatexDN  : t_write_phy_states := E_WRITE_PHY_IDLE;
        --     signal EastWritePhyStatexDP  : t_write_phy_states := E_WRITE_PHY_IDLE;
        --     -- South
        --     signal SouthReadPhyStatexDN  : t_read_phy_states  := E_READ_PHY_IDLE;
        --     signal SouthReadPhyStatexDP  : t_read_phy_states  := E_READ_PHY_IDLE;
        --     signal SouthWritePhyStatexDN : t_write_phy_states := E_WRITE_PHY_IDLE;
        --     signal SouthWritePhyStatexDP : t_write_phy_states := E_WRITE_PHY_IDLE;
        --     -- West
        --     signal WestReadPhyStatexDN   : t_read_phy_states  := E_READ_PHY_IDLE;
        --     signal WestReadPhyStatexDP   : t_read_phy_states  := E_READ_PHY_IDLE;
        --     signal WestWritePhyStatexDN  : t_write_phy_states := E_WRITE_PHY_IDLE;
        --     signal WestWritePhyStatexDP  : t_write_phy_states := E_WRITE_PHY_IDLE;

        --     attribute mark_debug                          : string;
        --     attribute keep                                : string;
        --     --
        --     attribute mark_debug of NorthWritePhyStatexDP : signal is "true";
        --     attribute keep of NorthWritePhyStatexDP       : signal is "true";
        --     -- attribute mark_debug of EastWritePhyStatexDP  : signal is "true";
        --     -- attribute keep of EastWritePhyStatexDP        : signal is "true";
        --     attribute mark_debug of SouthWritePhyStatexDP : signal is "true";
        --     attribute keep of SouthWritePhyStatexDP       : signal is "true";
        --     -- attribute mark_debug of WestWritePhyStatexDP  : signal is "true";
        --     -- attribute keep of WestWritePhyStatexDP        : signal is "true";
        --     attribute mark_debug of NorthReadPhyStatexDP  : signal is "true";
        --     attribute keep of NorthReadPhyStatexDP        : signal is "true";
        --     -- attribute mark_debug of EastReadPhyStatexDP  : signal is "true";
        --     -- attribute keep of EastReadPhyStatexDP        : signal is "true";
        --     attribute mark_debug of SouthReadPhyStatexDP  : signal is "true";
        --     attribute keep of SouthReadPhyStatexDP        : signal is "true";
        --     -- attribute mark_debug of WestReadPhyStatexDP  : signal is "true";
        --     -- attribute keep of WestReadPhyStatexDP        : signal is "true";

        -- begin  -- block DebugxB

        --     -- RX Fifo reset 196 cycles
        --     -- Clock and Resets
        --     -- RX Fifo
        --     NorthFifoResetxAS : RXResetxR.FifoResetxR.NorthxR <=
        --         '1' when
        --         (DebugRXFifoResetxR.NorthxR = '1') or
        --         (AuroraStatusxD.LaneUpxD(0)(0) = '0') or
        --         (AuroraStatusxD.ChannelUpxD(0) = '0') else
        --         '0';
        --     EastFifoResetxAS : RXResetxR.FifoResetxR.EastxR <=
        --         '1' when
        --         (DebugRXFifoResetxR.EastxR = '1') or
        --         (AuroraStatusxD.LaneUpxD(1)(0) = '0') or
        --         (AuroraStatusxD.ChannelUpxD(1) = '0') else
        --         '0';
        --     SouthFifoResetxAS : RXResetxR.FifoResetxR.SouthxR <=
        --         '1' when
        --         (DebugRXFifoResetxR.SouthxR = '1') or
        --         (AuroraStatusxD.LaneUpxD(2)(0) = '0') or
        --         (AuroraStatusxD.ChannelUpxD(2) = '0') else
        --         '0';
        --     WestFifoResetxAS : RXResetxR.FifoResetxR.WestxR <=
        --         '1' when
        --         (DebugRXFifoResetxR.WestxR = '1') or
        --         (AuroraStatusxD.LaneUpxD(3)(0) = '0') or
        --         (AuroraStatusxD.ChannelUpxD(3) = '0') else
        --         '0';
        --     -- Back pressure
        --     NorthBackPressureResetxAS : RXResetxR.BackPressureResetxR.NorthxR <=
        --         '1' when
        --         (DebugBackPressureResetxR.NorthxR = '1') or
        --         (AuroraStatusxD.LaneUpxD(0)(0) = '0') or
        --         (RXFifoResetDonexD.NorthxS = '0') or
        --         (RXFifoResetDoneDelayedxD.NorthxS = '0') or
        --         (AuroraStatusxD.ChannelUpxD(0) = '0') else
        --         '0';
        --     EastBackPressureResetxAS : RXResetxR.BackPressureResetxR.EastxR <=
        --         '1' when
        --         (DebugBackPressureResetxR.EastxR = '1') or
        --         (AuroraStatusxD.LaneUpxD(1)(0) = '0') or
        --         (RXFifoResetDonexD.EastxS = '0') or
        --         (RXFifoResetDoneDelayedxD.EastxS = '0') or
        --         (AuroraStatusxD.ChannelUpxD(1) = '0') else
        --         '0';
        --     SouthBackPressureResetxAS : RXResetxR.BackPressureResetxR.SouthxR <=
        --         '1' when
        --         (DebugBackPressureResetxR.SouthxR = '1') or
        --         (RXFifoResetDonexD.SouthxS = '0') or
        --         (RXFifoResetDoneDelayedxD.SouthxS = '0') or
        --         (AuroraStatusxD.LaneUpxD(2)(0) = '0') or
        --         (AuroraStatusxD.ChannelUpxD(2) = '0') else
        --         '0';
        --     WestBackPressureResetxAS : RXResetxR.BackPressureResetxR.WestxR <=
        --         '1' when
        --         (DebugBackPressureResetxR.WestxR = '1') or
        --         (AuroraStatusxD.LaneUpxD(3)(0) = '0') or
        --         (RXFifoResetDonexD.WestxS = '0') or
        --         (RXFifoResetDoneDelayedxD.WestxS = '0') or
        --         (AuroraStatusxD.ChannelUpxD(3) = '0') else
        --         '0';

        --     NorthUpdateRegxP : process (AuroraClkMasterxC.UserClkxC,
        --                                 AuroraStatusxD.ChannelUpxD(0),
        --                                 AuroraStatusxD.LaneUpxD(0)(0),
        --                                 DebugCounterResetxR.NorthxR,
        --                                 RXFifoResetDoneDelayedxD.NorthxS,
        --                                 RXFifoResetDonexD.NorthxS) is
        --     begin  -- process NorthUpdateRegxP
        --         if (DebugCounterResetxR.NorthxR = '1') or
        --             (AuroraStatusxD.LaneUpxD(0)(0) = '0') or
        --             (AuroraStatusxD.ChannelUpxD(0) = '0') or
        --             (RXFifoResetDonexD.NorthxS = '0') or
        --             (RXFifoResetDoneDelayedxD.NorthxS = '0') then
        --             NorthReadPhyStatexDP  <= E_READ_PHY_IDLE;
        --             NorthWritePhyStatexDP <= E_WRITE_PHY_IDLE;
        --             NorthDataCounterxDP   <= (others => '0');
        --         elsif rising_edge(AuroraClkMasterxC.UserClkxC) then
        --             NorthReadPhyStatexDP  <= NorthReadPhyStatexDN;
        --             NorthWritePhyStatexDP <= NorthWritePhyStatexDN;
        --             NorthDataCounterxDP   <= NorthDataCounterxDN;
        --         end if;
        --     end process NorthUpdateRegxP;

        --     EastUpdateRegxP : process (AuroraClkMasterxC.UserClkxC,
        --                                AuroraStatusxD.ChannelUpxD(1),
        --                                AuroraStatusxD.LaneUpxD(1)(0),
        --                                DebugCounterResetxR.EastxR,
        --                                RXFifoResetDoneDelayedxD.EastxS,
        --                                RXFifoResetDonexD.EastxS) is
        --     begin  -- process EastUpdateRegxP
        --         if (DebugCounterResetxR.EastxR = '1') or
        --             (AuroraStatusxD.LaneUpxD(1)(0) = '0') or
        --             (AuroraStatusxD.ChannelUpxD(1) = '0') or
        --             (RXFifoResetDonexD.EastxS = '0') or
        --             (RXFifoResetDoneDelayedxD.EastxS = '0') then
        --             EastReadPhyStatexDP  <= E_READ_PHY_IDLE;
        --             EastWritePhyStatexDP <= E_WRITE_PHY_IDLE;
        --             EastDataCounterxDP   <= (others => '0');
        --         elsif rising_edge(AuroraClkMasterxC.UserClkxC) then
        --             EastReadPhyStatexDP  <= EastReadPhyStatexDN;
        --             EastWritePhyStatexDP <= EastWritePhyStatexDN;
        --             EastDataCounterxDP   <= EastDataCounterxDN;
        --         end if;
        --     end process EastUpdateRegxP;

        --     SouthUpdateRegxP : process (AuroraClkMasterxC.UserClkxC,
        --                                 AuroraStatusxD.ChannelUpxD(2),
        --                                 AuroraStatusxD.LaneUpxD(2)(0),
        --                                 DebugCounterResetxR.SouthxR,
        --                                 RXFifoResetDoneDelayedxD.SouthxS,
        --                                 RXFifoResetDonexD.SouthxS) is
        --     begin  -- process SouthUpdateRegxP
        --         if (DebugCounterResetxR.SouthxR = '1') or
        --             (AuroraStatusxD.LaneUpxD(2)(0) = '0') or
        --             (AuroraStatusxD.ChannelUpxD(2) = '0') or
        --             (RXFifoResetDonexD.SouthxS = '0') or
        --             (RXFifoResetDoneDelayedxD.SouthxS = '0') then
        --             SouthReadPhyStatexDP  <= E_READ_PHY_IDLE;
        --             SouthWritePhyStatexDP <= E_WRITE_PHY_IDLE;
        --             SouthDataCounterxDP   <= (others => '0');
        --         elsif rising_edge(AuroraClkMasterxC.UserClkxC) then
        --             SouthReadPhyStatexDP  <= SouthReadPhyStatexDN;
        --             SouthWritePhyStatexDP <= SouthWritePhyStatexDN;
        --             SouthDataCounterxDP   <= SouthDataCounterxDN;
        --         end if;
        --     end process SouthUpdateRegxP;

        --     WestUpdateRegxP : process (AuroraClkMasterxC.UserClkxC,
        --                                AuroraStatusxD.ChannelUpxD(3),
        --                                AuroraStatusxD.LaneUpxD(3)(0),
        --                                DebugCounterResetxR.WestxR,
        --                                RXFifoResetDoneDelayedxD.WestxS,
        --                                RXFifoResetDonexD.WestxS) is
        --     begin  -- process WestUpdateRegxP
        --         if (DebugCounterResetxR.WestxR = '1') or
        --             (AuroraStatusxD.LaneUpxD(3)(0) = '0') or
        --             (AuroraStatusxD.ChannelUpxD(3) = '0') or
        --             (RXFifoResetDonexD.WestxS = '0') or
        --             (RXFifoResetDoneDelayedxD.WestxS = '0') then
        --             WestReadPhyStatexDP  <= E_READ_PHY_IDLE;
        --             WestWritePhyStatexDP <= E_WRITE_PHY_IDLE;
        --             WestDataCounterxDP   <= (others => '0');
        --         elsif rising_edge(AuroraClkMasterxC.UserClkxC) then
        --             WestReadPhyStatexDP  <= WestReadPhyStatexDN;
        --             WestWritePhyStatexDP <= WestWritePhyStatexDN;
        --             WestDataCounterxDP   <= WestDataCounterxDN;
        --         end if;
        --     end process WestUpdateRegxP;

        --     VioUserResetsxI : entity work.vio_user_resets
        --         port map (
        --             clk            => AuroraClkMasterxC.UserClkxC,
        --             probe_out0(0)  => DebugCounterResetxR.NorthxR,
        --             probe_out1(0)  => DebugCounterResetxR.EastxR,
        --             probe_out2(0)  => DebugCounterResetxR.SouthxR,
        --             probe_out3(0)  => DebugCounterResetxR.WestxR,
        --             probe_out4(0)  => DebugBackPressureResetxR.NorthxR,
        --             probe_out5(0)  => DebugBackPressureResetxR.EastxR,
        --             probe_out6(0)  => DebugBackPressureResetxR.SouthxR,
        --             probe_out7(0)  => DebugBackPressureResetxR.WestxR,
        --             probe_out8(0)  => DebugRXFifoResetxR.NorthxR,
        --             probe_out9(0)  => DebugRXFifoResetxR.EastxR,
        --             probe_out10(0) => DebugRXFifoResetxR.SouthxR,
        --             probe_out11(0) => DebugRXFifoResetxR.WestxR);

        --     VioStatusxI : entity work.vio_status
        --         port map (
        --             clk           => AuroraClkMasterxC.UserClkxC,
        --             -- North
        --             probe_in0(0)  => AuroraStatusxD.HardErrxD(0),
        --             probe_in1(0)  => AuroraStatusxD.SoftErrxD(0),
        --             probe_in2(0)  => AuroraStatusxD.FrameErrxD(0),
        --             probe_in3     => AuroraStatusxD.LaneUpxD(0),
        --             probe_in4(0)  => AuroraStatusxD.ChannelUpxD(0),
        --             probe_in5(0)  => AuroraStatusxD.RXResetDoneOutxD(0),
        --             probe_in6(0)  => AuroraStatusxD.TXResetDoneOutxD(0),
        --             -- East
        --             probe_in7(0)  => AuroraStatusxD.HardErrxD(1),
        --             probe_in8(0)  => AuroraStatusxD.SoftErrxD(1),
        --             probe_in9(0)  => AuroraStatusxD.FrameErrxD(1),
        --             probe_in10    => AuroraStatusxD.LaneUpxD(1),
        --             probe_in11(0) => AuroraStatusxD.ChannelUpxD(1),
        --             probe_in12(0) => AuroraStatusxD.RXResetDoneOutxD(1),
        --             probe_in13(0) => AuroraStatusxD.TXResetDoneOutxD(1),
        --             -- South
        --             probe_in14(0) => AuroraStatusxD.HardErrxD(2),
        --             probe_in15(0) => AuroraStatusxD.SoftErrxD(2),
        --             probe_in16(0) => AuroraStatusxD.FrameErrxD(2),
        --             probe_in17    => AuroraStatusxD.LaneUpxD(2),
        --             probe_in18(0) => AuroraStatusxD.ChannelUpxD(2),
        --             probe_in19(0) => AuroraStatusxD.RXResetDoneOutxD(2),
        --             probe_in20(0) => AuroraStatusxD.TXResetDoneOutxD(2),
        --             -- West
        --             probe_in21(0) => AuroraStatusxD.HardErrxD(3),
        --             probe_in22(0) => AuroraStatusxD.SoftErrxD(3),
        --             probe_in23(0) => AuroraStatusxD.FrameErrxD(3),
        --             probe_in24    => AuroraStatusxD.LaneUpxD(3),
        --             probe_in25(0) => AuroraStatusxD.ChannelUpxD(3),
        --             probe_in26(0) => AuroraStatusxD.RXResetDoneOutxD(3),
        --             probe_in27(0) => AuroraStatusxD.TXResetDoneOutxD(3));

        --     NorthWriteTXPhyxP : process (NorthDataCounterxDP,
        --                                  NorthTXS2MxD.ReadyxS,
        --                                  NorthWritePhyStatexDP) is
        --     begin  -- process NorthWriteTXPhyxP
        --         -- Default values
        --         NorthWritePhyStatexDN <= NorthWritePhyStatexDP;
        --         NorthDataCounterxDN   <= NorthDataCounterxDP;
        --         NorthTXM2SxD.DataxD   <= (others => '0');
        --         NorthTXM2SxD.KeepxD   <= (others => '1');
        --         NorthTXM2SxD.ValidxS  <= '0';
        --         NorthTXM2SxD.LastxS   <= '0';

        --         case NorthWritePhyStatexDP is
        --             when E_WRITE_PHY_IDLE =>
        --                     NorthWritePhyStatexDN <= E_WRITE_PHY_W0;

        --             when E_WRITE_PHY_W0 =>
        --                     NorthTXM2SxD.DataxD  <= std_logic_vector(NorthDataCounterxDP);
        --                     NorthTXM2SxD.ValidxS <= '1';

        --                     if NorthTXS2MxD.ReadyxS = '1' then
        --                         NorthWritePhyStatexDN <= E_WRITE_PHY_W1;
        --                     end if;

        --             when E_WRITE_PHY_W1 =>
        --                     NorthTXM2SxD.DataxD  <= std_logic_vector(NorthDataCounterxDP);
        --                     NorthTXM2SxD.ValidxS <= '1';

        --                     if NorthTXS2MxD.ReadyxS = '1' then
        --                         NorthWritePhyStatexDN <= E_WRITE_PHY_W2;
        --                     end if;

        --             when E_WRITE_PHY_W2 =>
        --                     NorthTXM2SxD.DataxD  <= std_logic_vector(NorthDataCounterxDP);
        --                     NorthTXM2SxD.ValidxS <= '1';

        --                     if NorthTXS2MxD.ReadyxS = '1' then
        --                         NorthWritePhyStatexDN <= E_WRITE_PHY_W3;
        --                     end if;

        --             when E_WRITE_PHY_W3 =>
        --                     NorthTXM2SxD.DataxD  <= std_logic_vector(NorthDataCounterxDP);
        --                     NorthTXM2SxD.ValidxS <= '1';

        --                     if NorthTXS2MxD.ReadyxS = '1' then
        --                         NorthWritePhyStatexDN <= E_WRITE_PHY_W4;
        --                     end if;

        --             when E_WRITE_PHY_W4 =>
        --                     NorthTXM2SxD.DataxD  <= std_logic_vector(NorthDataCounterxDP);
        --                     NorthTXM2SxD.ValidxS <= '1';

        --                     if NorthTXS2MxD.ReadyxS = '1' then
        --                         NorthWritePhyStatexDN <= E_WRITE_PHY_W5;
        --                     end if;

        --             when E_WRITE_PHY_W5 =>
        --                     NorthTXM2SxD.DataxD  <= std_logic_vector(NorthDataCounterxDP);
        --                     NorthTXM2SxD.ValidxS <= '1';

        --                     if NorthTXS2MxD.ReadyxS = '1' then
        --                         NorthWritePhyStatexDN <= E_WRITE_PHY_W6;
        --                     end if;
        --             when E_WRITE_PHY_W6 =>
        --                     NorthTXM2SxD.DataxD  <= std_logic_vector(NorthDataCounterxDP);
        --                     NorthTXM2SxD.ValidxS <= '1';

        --                     if NorthTXS2MxD.ReadyxS = '1' then
        --                         NorthWritePhyStatexDN <= E_WRITE_PHY_W7;
        --                     end if;

        --             when E_WRITE_PHY_W7 =>
        --                     NorthTXM2SxD.DataxD  <= std_logic_vector(NorthDataCounterxDP);
        --                     NorthTXM2SxD.ValidxS <= '1';
        --                     NorthTXM2SxD.LastxS  <= '1';

        --                     if NorthTXS2MxD.ReadyxS = '1' then
        --                         NorthWritePhyStatexDN <= E_WRITE_PHY_W0;
        --                         NorthDataCounterxDN   <= NorthDataCounterxDP + 1;
        --                     end if;

        --             when others => null;
        --         end case;

        --     end process NorthWriteTXPhyxP;

        --     EastWriteTXPhyxP : process (EastDataCounterxDP,
        --                                 EastTXS2MxD.ReadyxS,
        --                                 EastWritePhyStatexDP) is
        --     begin  -- process EastWriteTXPhyxP
        --         -- Default values
        --         EastWritePhyStatexDN <= EastWritePhyStatexDP;
        --         EastDataCounterxDN   <= EastDataCounterxDP;
        --         EastTXM2SxD.DataxD   <= (others => '0');
        --         EastTXM2SxD.KeepxD   <= (others => '1');
        --         EastTXM2SxD.ValidxS  <= '0';
        --         EastTXM2SxD.LastxS   <= '0';

        --         case EastWritePhyStatexDP is
        --             when E_WRITE_PHY_IDLE =>
        --                     EastWritePhyStatexDN <= E_WRITE_PHY_W0;

        --             when E_WRITE_PHY_W0 =>
        --                     EastTXM2SxD.DataxD  <= std_logic_vector(EastDataCounterxDP);
        --                     EastTXM2SxD.ValidxS <= '1';

        --                     if EastTXS2MxD.ReadyxS = '1' then
        --                         EastWritePhyStatexDN <= E_WRITE_PHY_W1;
        --                     end if;

        --             when E_WRITE_PHY_W1 =>
        --                     EastTXM2SxD.DataxD  <= std_logic_vector(EastDataCounterxDP);
        --                     EastTXM2SxD.ValidxS <= '1';

        --                     if EastTXS2MxD.ReadyxS = '1' then
        --                         EastWritePhyStatexDN <= E_WRITE_PHY_W2;
        --                     end if;

        --             when E_WRITE_PHY_W2 =>
        --                     EastTXM2SxD.DataxD  <= std_logic_vector(EastDataCounterxDP);
        --                     EastTXM2SxD.ValidxS <= '1';

        --                     if EastTXS2MxD.ReadyxS = '1' then
        --                         EastWritePhyStatexDN <= E_WRITE_PHY_W3;
        --                     end if;

        --             when E_WRITE_PHY_W3 =>
        --                     EastTXM2SxD.DataxD  <= std_logic_vector(EastDataCounterxDP);
        --                     EastTXM2SxD.ValidxS <= '1';

        --                     if EastTXS2MxD.ReadyxS = '1' then
        --                         EastWritePhyStatexDN <= E_WRITE_PHY_W4;
        --                     end if;

        --             when E_WRITE_PHY_W4 =>
        --                     EastTXM2SxD.DataxD  <= std_logic_vector(EastDataCounterxDP);
        --                     EastTXM2SxD.ValidxS <= '1';

        --                     if EastTXS2MxD.ReadyxS = '1' then
        --                         EastWritePhyStatexDN <= E_WRITE_PHY_W5;
        --                     end if;

        --             when E_WRITE_PHY_W5 =>
        --                     EastTXM2SxD.DataxD  <= std_logic_vector(EastDataCounterxDP);
        --                     EastTXM2SxD.ValidxS <= '1';

        --                     if EastTXS2MxD.ReadyxS = '1' then
        --                         EastWritePhyStatexDN <= E_WRITE_PHY_W6;
        --                     end if;

        --             when E_WRITE_PHY_W6 =>
        --                     EastTXM2SxD.DataxD  <= std_logic_vector(EastDataCounterxDP);
        --                     EastTXM2SxD.ValidxS <= '1';

        --                     if EastTXS2MxD.ReadyxS = '1' then
        --                         EastWritePhyStatexDN <= E_WRITE_PHY_W7;
        --                     end if;

        --             when E_WRITE_PHY_W7 =>
        --                     EastTXM2SxD.DataxD  <= std_logic_vector(EastDataCounterxDP);
        --                     EastTXM2SxD.ValidxS <= '1';
        --                     EastTXM2SxD.LastxS  <= '1';

        --                     if EastTXS2MxD.ReadyxS = '1' then
        --                         EastWritePhyStatexDN <= E_WRITE_PHY_W0;
        --                         EastDataCounterxDN   <= EastDataCounterxDP + 1;
        --                     end if;

        --             when others => null;
        --         end case;

        --     end process EastWriteTXPhyxP;

        --     SouthWriteTXPhyxP : process (SouthDataCounterxDP,
        --                                  SouthTXS2MxD.ReadyxS,
        --                                  SouthWritePhyStatexDP) is
        --     begin  -- process SouthWriteTXPhyxP
        --         -- Default values
        --         SouthWritePhyStatexDN <= SouthWritePhyStatexDP;
        --         SouthDataCounterxDN   <= SouthDataCounterxDP;
        --         SouthTXM2SxD.DataxD   <= (others => '0');
        --         SouthTXM2SxD.KeepxD   <= (others => '1');
        --         SouthTXM2SxD.ValidxS  <= '0';
        --         SouthTXM2SxD.LastxS   <= '0';

        --         case SouthWritePhyStatexDP is
        --             when E_WRITE_PHY_IDLE =>
        --                     SouthWritePhyStatexDN <= E_WRITE_PHY_W0;

        --             when E_WRITE_PHY_W0 =>
        --                     SouthTXM2SxD.DataxD  <= std_logic_vector(SouthDataCounterxDP);
        --                     SouthTXM2SxD.ValidxS <= '1';

        --                     if SouthTXS2MxD.ReadyxS = '1' then
        --                         SouthWritePhyStatexDN <= E_WRITE_PHY_W1;
        --                     end if;

        --             when E_WRITE_PHY_W1 =>
        --                     SouthTXM2SxD.DataxD  <= std_logic_vector(SouthDataCounterxDP);
        --                     SouthTXM2SxD.ValidxS <= '1';

        --                     if SouthTXS2MxD.ReadyxS = '1' then
        --                         SouthWritePhyStatexDN <= E_WRITE_PHY_W2;
        --                     end if;

        --             when E_WRITE_PHY_W2 =>
        --                     SouthTXM2SxD.DataxD  <= std_logic_vector(SouthDataCounterxDP);
        --                     SouthTXM2SxD.ValidxS <= '1';

        --                     if SouthTXS2MxD.ReadyxS = '1' then
        --                         SouthWritePhyStatexDN <= E_WRITE_PHY_W3;
        --                     end if;

        --             when E_WRITE_PHY_W3 =>
        --                     SouthTXM2SxD.DataxD  <= std_logic_vector(SouthDataCounterxDP);
        --                     SouthTXM2SxD.ValidxS <= '1';

        --                     if SouthTXS2MxD.ReadyxS = '1' then
        --                         SouthWritePhyStatexDN <= E_WRITE_PHY_W4;
        --                     end if;

        --             when E_WRITE_PHY_W4 =>
        --                     SouthTXM2SxD.DataxD  <= std_logic_vector(SouthDataCounterxDP);
        --                     SouthTXM2SxD.ValidxS <= '1';

        --                     if SouthTXS2MxD.ReadyxS = '1' then
        --                         SouthWritePhyStatexDN <= E_WRITE_PHY_W5;
        --                     end if;

        --             when E_WRITE_PHY_W5 =>
        --                     SouthTXM2SxD.DataxD  <= std_logic_vector(SouthDataCounterxDP);
        --                     SouthTXM2SxD.ValidxS <= '1';

        --                     if SouthTXS2MxD.ReadyxS = '1' then
        --                         SouthWritePhyStatexDN <= E_WRITE_PHY_W6;
        --                     end if;

        --             when E_WRITE_PHY_W6 =>
        --                     SouthTXM2SxD.DataxD  <= std_logic_vector(SouthDataCounterxDP);
        --                     SouthTXM2SxD.ValidxS <= '1';

        --                     if SouthTXS2MxD.ReadyxS = '1' then
        --                         SouthWritePhyStatexDN <= E_WRITE_PHY_W7;
        --                     end if;

        --             when E_WRITE_PHY_W7 =>
        --                     SouthTXM2SxD.DataxD  <= std_logic_vector(SouthDataCounterxDP);
        --                     SouthTXM2SxD.ValidxS <= '1';
        --                     SouthTXM2SxD.LastxS  <= '1';

        --                     if SouthTXS2MxD.ReadyxS = '1' then
        --                         SouthWritePhyStatexDN <= E_WRITE_PHY_W0;
        --                         SouthDataCounterxDN   <= SouthDataCounterxDP + 1;
        --                     end if;

        --             when others => null;
        --         end case;

        --     end process SouthWriteTXPhyxP;

        --     WestWriteTXPhyxP : process (WestDataCounterxDP,
        --                                 WestTXS2MxD.ReadyxS,
        --                                 WestWritePhyStatexDP) is
        --     begin  -- process WestWriteTXPhyxP
        --         -- Default values
        --         WestWritePhyStatexDN <= WestWritePhyStatexDP;
        --         WestDataCounterxDN   <= WestDataCounterxDP;
        --         WestTXM2SxD.DataxD   <= (others => '0');
        --         WestTXM2SxD.KeepxD   <= (others => '1');
        --         WestTXM2SxD.ValidxS  <= '0';
        --         WestTXM2SxD.LastxS   <= '0';

        --         case WestWritePhyStatexDP is
        --             when E_WRITE_PHY_IDLE =>
        --                     WestWritePhyStatexDN <= E_WRITE_PHY_W0;

        --             when E_WRITE_PHY_W0 =>
        --                     WestTXM2SxD.DataxD  <= std_logic_vector(WestDataCounterxDP);
        --                     WestTXM2SxD.ValidxS <= '1';

        --                     if WestTXS2MxD.ReadyxS = '1' then
        --                         WestWritePhyStatexDN <= E_WRITE_PHY_W1;
        --                     end if;

        --             when E_WRITE_PHY_W1 =>
        --                     WestTXM2SxD.DataxD  <= std_logic_vector(WestDataCounterxDP);
        --                     WestTXM2SxD.ValidxS <= '1';

        --                     if WestTXS2MxD.ReadyxS = '1' then
        --                         WestWritePhyStatexDN <= E_WRITE_PHY_W2;
        --                     end if;

        --             when E_WRITE_PHY_W2 =>
        --                     WestTXM2SxD.DataxD  <= std_logic_vector(WestDataCounterxDP);
        --                     WestTXM2SxD.ValidxS <= '1';

        --                     if WestTXS2MxD.ReadyxS = '1' then
        --                         WestWritePhyStatexDN <= E_WRITE_PHY_W3;
        --                     end if;

        --             when E_WRITE_PHY_W3 =>
        --                     WestTXM2SxD.DataxD  <= std_logic_vector(WestDataCounterxDP);
        --                     WestTXM2SxD.ValidxS <= '1';

        --                     if WestTXS2MxD.ReadyxS = '1' then
        --                         WestWritePhyStatexDN <= E_WRITE_PHY_W4;
        --                     end if;

        --             when E_WRITE_PHY_W4 =>
        --                     WestTXM2SxD.DataxD  <= std_logic_vector(WestDataCounterxDP);
        --                     WestTXM2SxD.ValidxS <= '1';

        --                     if WestTXS2MxD.ReadyxS = '1' then
        --                         WestWritePhyStatexDN <= E_WRITE_PHY_W5;
        --                     end if;

        --             when E_WRITE_PHY_W5 =>
        --                     WestTXM2SxD.DataxD  <= std_logic_vector(WestDataCounterxDP);
        --                     WestTXM2SxD.ValidxS <= '1';

        --                     if WestTXS2MxD.ReadyxS = '1' then
        --                         WestWritePhyStatexDN <= E_WRITE_PHY_W6;
        --                     end if;

        --             when E_WRITE_PHY_W6 =>
        --                     WestTXM2SxD.DataxD  <= std_logic_vector(WestDataCounterxDP);
        --                     WestTXM2SxD.ValidxS <= '1';

        --                     if WestTXS2MxD.ReadyxS = '1' then
        --                         WestWritePhyStatexDN <= E_WRITE_PHY_W7;
        --                     end if;

        --             when E_WRITE_PHY_W7 =>
        --                     WestTXM2SxD.DataxD  <= std_logic_vector(WestDataCounterxDP);
        --                     WestTXM2SxD.ValidxS <= '1';
        --                     WestTXM2SxD.LastxS  <= '1';

        --                     if WestTXS2MxD.ReadyxS = '1' then
        --                         WestWritePhyStatexDN <= E_WRITE_PHY_W0;
        --                         WestDataCounterxDN   <= WestDataCounterxDP + 1;
        --                     end if;

        --             when others => null;
        --         end case;

        --     end process WestWriteTXPhyxP;

        --     NorthReadRXPhyxP : process (NorthRXFifoStatusxD.ProgEmptyxS,
        --                                 NorthRXFifoStatusxD.ProgFullxS,
        --                                 NorthRXM2SxD.ValidxS,
        --                                 NorthReadPhyStatexDP) is
        --     begin  -- process NorthReadRXPhyxP
        --         -- Default value
        --         NorthReadPhyStatexDN <= NorthReadPhyStatexDP;
        --         NorthRXS2MxD.ReadyxS <= '0';

        --         case NorthReadPhyStatexDP is
        --             when E_READ_PHY_IDLE =>
        --                     if (NorthRXFifoStatusxD.ProgFullxS = '1') and
        --                         (NorthRXFifoStatusxD.ProgEmptyxS = '0') then
        --                         NorthReadPhyStatexDN <= E_READ_PHY_S0;
        --                     end if;

        --             when E_READ_PHY_S0 =>
        --                     if NorthRXM2SxD.ValidxS = '1' then
        --                         NorthRXS2MxD.ReadyxS <= '1';
        --                     end if;

        --                     if (NorthRXFifoStatusxD.ProgFullxS = '0') and
        --                         (NorthRXFifoStatusxD.ProgEmptyxS = '1') then
        --                         NorthReadPhyStatexDN <= E_READ_PHY_IDLE;
        --                     end if;

        --             when others => null;
        --         end case;
        --     end process NorthReadRXPhyxP;

        --     EastReadRXPhyxP : process (EastRXFifoStatusxD.ProgEmptyxS,
        --                                EastRXFifoStatusxD.ProgFullxS,
        --                                EastRXM2SxD.ValidxS,
        --                                EastReadPhyStatexDP) is
        --     begin  -- process EastReadRXPhyxP
        --         -- Default value
        --         EastReadPhyStatexDN <= EastReadPhyStatexDP;
        --         EastRXS2MxD.ReadyxS <= '0';

        --         case EastReadPhyStatexDP is
        --             when E_READ_PHY_IDLE =>
        --                     if (EastRXFifoStatusxD.ProgFullxS = '1') and
        --                         (EastRXFifoStatusxD.ProgEmptyxS = '0') then
        --                         EastReadPhyStatexDN <= E_READ_PHY_S0;
        --                     end if;

        --             when E_READ_PHY_S0 =>
        --                     if EastRXM2SxD.ValidxS = '1' then
        --                         EastRXS2MxD.ReadyxS <= '1';
        --                     end if;

        --                     if (EastRXFifoStatusxD.ProgFullxS = '0') and
        --                         (EastRXFifoStatusxD.ProgEmptyxS = '1') then
        --                         EastReadPhyStatexDN <= E_READ_PHY_IDLE;
        --                     end if;

        --             when others => null;
        --         end case;
        --     end process EastReadRXPhyxP;

        --     SouthReadRXPhyxP : process (SouthRXFifoStatusxD.ProgEmptyxS,
        --                                 SouthRXFifoStatusxD.ProgFullxS,
        --                                 SouthRXM2SxD.ValidxS,
        --                                 SouthReadPhyStatexDP) is
        --     begin  -- process SouthReadRXPhyxP
        --         -- Default value
        --         SouthReadPhyStatexDN <= SouthReadPhyStatexDP;
        --         SouthRXS2MxD.ReadyxS <= '0';

        --         case SouthReadPhyStatexDP is
        --             when E_READ_PHY_IDLE =>
        --                     if (SouthRXFifoStatusxD.ProgFullxS = '1') and
        --                         (SouthRXFifoStatusxD.ProgEmptyxS = '0') then
        --                         SouthReadPhyStatexDN <= E_READ_PHY_S0;
        --                     end if;

        --             when E_READ_PHY_S0 =>
        --                     if SouthRXM2SxD.ValidxS = '1' then
        --                         SouthRXS2MxD.ReadyxS <= '1';
        --                     end if;

        --                     if (SouthRXFifoStatusxD.ProgFullxS = '0') and
        --                         (SouthRXFifoStatusxD.ProgEmptyxS = '1') then
        --                         SouthReadPhyStatexDN <= E_READ_PHY_IDLE;
        --                     end if;

        --             when others => null;
        --         end case;
        --     end process SouthReadRXPhyxP;

        --     WestReadRXPhyxP : process (WestRXFifoStatusxD.ProgEmptyxS,
        --                                WestRXFifoStatusxD.ProgFullxS,
        --                                WestRXM2SxD.ValidxS,
        --                                WestReadPhyStatexDP) is
        --     begin  -- process WestReadRXPhyxP
        --         -- Default value
        --         WestReadPhyStatexDN <= WestReadPhyStatexDP;
        --         WestRXS2MxD.ReadyxS <= '0';

        --         case WestReadPhyStatexDP is
        --             when E_READ_PHY_IDLE =>
        --                     if (WestRXFifoStatusxD.ProgFullxS = '1') and
        --                         (WestRXFifoStatusxD.ProgEmptyxS = '0') then
        --                         WestReadPhyStatexDN <= E_READ_PHY_S0;
        --                     end if;

        --             when E_READ_PHY_S0 =>
        --                     if WestRXM2SxD.ValidxS = '1' then
        --                         WestRXS2MxD.ReadyxS <= '1';
        --                     end if;

        --                     if (WestRXFifoStatusxD.ProgFullxS = '0') and
        --                         (WestRXFifoStatusxD.ProgEmptyxS = '1') then
        --                         WestReadPhyStatexDN <= E_READ_PHY_IDLE;
        --                     end if;

        --             when others => null;
        --         end case;
        --     end process WestReadRXPhyxP;

        -- end block DebugxB;

    end block ProgrammableLogicxB;

end arch;
