----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: tb_scalp_router_core.vhd
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.1
-- Description: Testbench for scalp_router_core.
--
-- Last update: 2020-01-21
--
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;
-- use ieee.math_real.uniform;
-- use ieee.math_real.floor;

--use ieee.std_logic_unsigned.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_misc.all;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

library work;
-- use work.aurora_axi4_pkg.all;
use work.scalp_misc.all;
use work.scalp_utility.all;

entity tb_scalp_router_core is

end entity tb_scalp_router_core;

architecture testbench of tb_scalp_router_core is

    constant C_CLK_PERIOD                  : time                   := 10 ns;
    -- Scalp Router Core
    constant C_SCALP_INTERFACE_VECTOR_SIZE : integer range 0 to 255 := 7;
    -- Router Net Address
    constant C_SCALP_LOC_ROUTER_NET_ADDR_0 : t_scalp_netaddr        := (XxD => 0, YxD => 0, ZxD => 0);
    constant C_SCALP_LOC_ROUTER_NET_ADDR_1 : t_scalp_netaddr        := (XxD => 1, YxD => 0, ZxD => 0);
    constant C_SCALP_LOC_ROUTER_NET_ADDR_2 : t_scalp_netaddr        := (XxD => 0, YxD => 1, ZxD => 0);
    constant C_SCALP_LOC_ROUTER_NET_ADDR_3 : t_scalp_netaddr        := (XxD => 1, YxD => 1, ZxD => 0);
    constant C_SCALP_DST_ROUTER_NET_ADDR_0 : t_scalp_netaddr        := (XxD => 1, YxD => 1, ZxD => 1);

    component scalp_router_core is
        generic (
            C_SCALP_INTERFACE_VECTOR_SIZE : integer range 0 to 255);
        port (
            -- System Clock and Reset
            SysClkxCI             : in  std_ulogic;
            SysRstxRNAI           : in  std_ulogic;
            LocRouterNetAddrxDI   : in  t_scalp_netaddr;
            DstRouterNetAddrxDI   : in  t_scalp_netaddr;
            RouterNetAddrValidxSI : in  std_ulogic;
            IfDstNumxDO           : out integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1);
            IfNumValidxSO         : out std_ulogic;
            QoSVectorxDI          : in  t_scalp_qos_vector((C_SCALP_INTERFACE_VECTOR_SIZE - 1) downto 0));
    end component scalp_router_core;

    -- Sys Clk and Sys nRst
    signal SysClkxC             : std_ulogic                                                       := '0';
    signal SysRstxRNA           : std_ulogic                                                       := '0';
    -- End of Simulation
    signal EndSimxS             : boolean                                                          := false;
    -- Scalp Router Core Signals
    signal LocRouterNetAddrxD   : t_scalp_netaddr                                                  := C_3D_MIN_SCALP_NETADDR;
    signal DstRouterNetAddrxD   : t_scalp_netaddr                                                  := C_3D_MIN_SCALP_NETADDR;
    signal RouterNetAddrValidxS : std_ulogic                                                       := '0';
    signal IfDstNumxD           : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1)           := 0;
    signal IfNumValidxS         : std_ulogic                                                       := '0';
    signal QoSVectorxD          : t_scalp_qos_vector((C_SCALP_INTERFACE_VECTOR_SIZE - 1) downto 0) := (others => C_SCALP_NO_QOS);

begin  -- architecture testbench

    AsyncStatementsxB : block is
    begin  -- block AsyncStatementsxB

        SysClkxAS : SysClkxC   <= not SysClkxC after (C_CLK_PERIOD / 2);
        SysRstxAS : SysRstxRNA <= '1'          after (C_CLK_PERIOD * 10);

    end block AsyncStatementsxB;

    EndSimxP : process is
    begin  -- process EndSimxP
        if EndSimxS = true then
            assert false report "End of simulation!" severity failure;
        end if;

        wait until rising_edge(SysClkxC);
    end process EndSimxP;

    ScalpRouterCorexI : entity work.scalp_router_core
        generic map (
            C_SCALP_INTERFACE_VECTOR_SIZE => C_SCALP_INTERFACE_VECTOR_SIZE)
        port map (
            -- System Clock and Reset
            SysClkxCI             => SysClkxC,
            SysRstxRNAI           => SysRstxRNA,
            -- Local and Destination Router Network Address
            LocRouterNetAddrxDI   => LocRouterNetAddrxD,
            DstRouterNetAddrxDI   => DstRouterNetAddrxD,
            RouterNetAddrValidxSI => RouterNetAddrValidxS,
            -- Destination Interfaces Number
            IfDstNumxDO           => IfDstNumxD,
            IfNumValidxSO         => IfNumValidxS,
            -- QoS Vector
            QoSVectorxDI          => QoSVectorxD);

    RouterNetAddrValidxP : process is
    begin  -- process RouterNetAddrValidxP

        wait until (SysRstxRNA = '1');
        wait until rising_edge(SysClkxC);
        wait until rising_edge(SysClkxC);

        for i in 0 to 1 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        RouterNetAddrValidxS <= '1';

        for i in 0 to 0 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        RouterNetAddrValidxS <= '0';

        wait;

    end process RouterNetAddrValidxP;

    ScalpRouterCorexP : process is
    begin  -- process ScalpRouterCorexP

        wait until (SysRstxRNA = '1');
        wait until rising_edge(SysClkxC);
        wait until rising_edge(SysClkxC);

        LocRouterNetAddrxD <= C_SCALP_LOC_ROUTER_NET_ADDR_3;
        DstRouterNetAddrxD <= C_SCALP_DST_ROUTER_NET_ADDR_0;

        wait until IfNumValidxS = '1';
        wait until IfNumValidxS = '0';

        EndSimxS <= true;

    end process ScalpRouterCorexP;

end architecture testbench;
